#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "data_entry.h"
#include <unistd.h>
#include <libpq-fe.h>
#include <gdk/gdkkeysyms.h>
#include "pizza_utils.h"

DataEntry *maint_cat_data_entry = NULL;

/*----------------------------------------------------------------------------------------------------
::cat_maintenance functions
----------------------------------------------------------------------------------------------------*/

/*----
	[list of categories]

	code [   ]
	desc [   ]  
	[allow discount]

	<ins> <delete> <update> <quit>

-----*/

enum {
	MAINT_CAT_LIST,
	MAINT_CAT_CODE,
	MAINT_CAT_DESC,
	MAINT_CAT_DISC,

	MAINT_CAT_INS_BTN,
	MAINT_CAT_DEL_BTN,
	MAINT_CAT_MOD_BTN,
	MAINT_CAT_QUIT_BTN
};

static void cat_list_populate (GtkCList *cat_list);

static void maint_cat_callback (GtkWidget *widget, gpointer data)
{
	DataEntry *data_entry;
	GtkInput *code, *desc;
	GtkToggleButton *disc, *insert_btn;
	GtkCList *cat_list;
	int btn_num;
	char command[255];

	data_entry = (DataEntry *) data;
	btn_num = data_entry_get_widget_id (data_entry, widget);

	cat_list = GTK_CLIST(data_entry_get_widget (data_entry, MAINT_CAT_LIST));
	code = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_CAT_CODE));
	desc = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_CAT_DESC));
	disc = GTK_TOGGLE_BUTTON(data_entry_get_widget (data_entry, MAINT_CAT_DISC));
	insert_btn = GTK_TOGGLE_BUTTON(data_entry_get_widget (data_entry, MAINT_CAT_INS_BTN));

	switch (btn_num)
	{
		case MAINT_CAT_MOD_BTN:
			if (insert_btn->active)
			{
				sprintf (command, "insert into category values ('%s','%s','%c')",
					gtk_input_get_text (code), 
					gtk_input_get_text (desc),
					(disc->active) ? 'f' : 't');
			}
			else // modify
			{
				sprintf (command, "update category set code = '%s', description = '%s', allow_discount = '%c' "
						"where code = '%s'",
					gtk_input_get_text (code), 
					gtk_input_get_text (desc),
					(disc->active) ? 'f' : 't',
					(char *) gtk_object_get_user_data (GTK_OBJECT(code)));
			}
			printf ("Command = '%s'\n", command);
			res = PQexec (conn, command);
			PQclear (res);
			gtk_input_set_text (code, "");
			gtk_input_set_text (desc, "");
			gtk_toggle_button_set_state (disc, FALSE);
			gtk_widget_grab_focus (GTK_WIDGET(cat_list));
			gtk_toggle_button_set_state (insert_btn, FALSE);
			cat_list_populate (cat_list);
			break;

		case MAINT_CAT_INS_BTN:
			gtk_widget_grab_focus (GTK_WIDGET(code));
			break;

		case MAINT_CAT_DEL_BTN:
			// delete the selected record
			sprintf (command, "delete from category where code = '%s'", 
				gtk_input_get_text (code));
			printf ("Command = '%s'\n", command);
			res = PQexec (conn, command);
			PQclear (res);
			gtk_input_set_text (code, "");
			gtk_input_set_text (desc, "");
			gtk_toggle_button_set_state (disc, FALSE);
			cat_list_populate (cat_list);
			gtk_widget_grab_focus (GTK_WIDGET(cat_list));
			gtk_toggle_button_set_state (insert_btn, FALSE);
			break;

		case MAINT_CAT_QUIT_BTN:
			gtk_widget_hide (GTK_WIDGET(data_entry));
			break;
	}
}

static void cat_list_populate (GtkCList *cat_list)
{
	int i, nTuples;
	char *cat_list_row[3];

	gtk_clist_freeze (cat_list);
	gtk_clist_clear (cat_list);
    res = PQexec(conn, "select code,description,allow_discount from category order by code");
    if (PQresultStatus(res) != PGRES_TUPLES_OK)
    {
        fprintf(stderr, "\nSelect didn't return tuples properly\n");
        PQclear(res);
        exit_nicely(conn);
    }

	nTuples = PQntuples (res);
	for (i = 0; i < nTuples; i++)
	{
		cat_list_row[0] = PQgetvalue (res, i, 0);
		cat_list_row[1] = PQgetvalue (res, i, 1);
		cat_list_row[2] = PQgetvalue (res, i, 2);
		gtk_clist_append (GTK_CLIST(cat_list), cat_list_row);
	}
	gtk_clist_thaw (cat_list);
	PQclear (res);
}

static void maint_cat_list_select (GtkWidget *widget, gint row, gint col, GdkEventButton *event, gpointer data)
{
	DataEntry *data_entry;
	GtkInput *code, *desc;
	GtkToggleButton *disc, *insert_btn;
	gchar *text;

	g_return_if_fail (widget);
	g_return_if_fail (data);
	data_entry = DATA_ENTRY (data);

	code = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_CAT_CODE));
	desc = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_CAT_DESC));
	disc = GTK_TOGGLE_BUTTON(data_entry_get_widget (data_entry, MAINT_CAT_DISC));
	insert_btn = GTK_TOGGLE_BUTTON(data_entry_get_widget (data_entry, MAINT_CAT_INS_BTN));

	gtk_clist_get_text (GTK_CLIST(widget), row, 0, &text);
	gtk_input_set_text (code, text);
	gtk_object_set_user_data (GTK_OBJECT(code), text);
	gtk_clist_get_text (GTK_CLIST(widget), row, 1, &text);
	gtk_input_set_text (desc, text);
	gtk_clist_get_text (GTK_CLIST(widget), row, 2, &text);
	gtk_toggle_button_set_state (disc, *text == 'f');
	gtk_widget_grab_focus (GTK_WIDGET(code));

	gtk_toggle_button_set_state (insert_btn, FALSE);
}

// called when the allow discount btn is pressed
static void maint_cat_disc_toggle (GtkToggleButton *widget)
{
	GList *children;
	GtkLabel *label;

	children = gtk_container_children (GTK_CONTAINER(widget));
	label = (GtkLabel *)(children->data);

	if (widget->active)
	{
		gtk_label_set (label, "No Discount");
	}
	else
	{
		gtk_label_set (label, "Allow Discount");
	}
}

void maint_cats (GtkWidget *widget, gpointer data)
{
	DataEntry *data_entry;
	static GtkWidget *cat_list, *code, *delbtn, *disc;
	GtkAcceleratorTable *accel;
	static char *titles[] = 
	{
		"Category", "Description", "Allow Discount"
	};

	if (!maint_cat_data_entry)
	{
		maint_cat_data_entry = DATA_ENTRY(data_entry_new ("Cateogries", 20, 140, 0,0));
		data_entry = maint_cat_data_entry;

		accel = gtk_accelerator_table_new ();
		gtk_window_add_accelerator_table (GTK_WINDOW(data_entry), accel);

		cat_list = gtk_clist_new_with_titles (3, titles);
		gtk_clist_set_column_width (GTK_CLIST(cat_list), 0, 100);
		gtk_clist_set_column_width (GTK_CLIST(cat_list), 1, 200);
		gtk_widget_set_usize (GTK_WIDGET(cat_list), 450, 200);
		gtk_signal_connect (GTK_OBJECT (cat_list), "select_row", 
			(GtkSignalFunc) maint_cat_list_select, data_entry);

		data_entry_add_widget (data_entry, MAINT_CAT_LIST, cat_list, 0,0,10,1);

		data_entry_add_label_button (data_entry, MAINT_CAT_INS_BTN, "Insert", 
				GTK_SIGNAL_FUNC(maint_cat_callback), BUTTON_NORMAL | BUTTON_TOGGLE);

		delbtn = data_entry_add_label_button (data_entry, MAINT_CAT_DEL_BTN, "Delete", 
				GTK_SIGNAL_FUNC(maint_cat_callback), BUTTON_NORMAL);
		gtk_accelerator_table_install (accel, GTK_OBJECT(delbtn), "clicked", '', 0);

		data_entry_add_label_button (data_entry, MAINT_CAT_MOD_BTN, "Update", 
				GTK_SIGNAL_FUNC(maint_cat_callback), BUTTON_DEFAULT);
		data_entry_add_label_button (data_entry, MAINT_CAT_QUIT_BTN, "Quit", 
				GTK_SIGNAL_FUNC(maint_cat_callback), BUTTON_QUIT);

		// now add the fields
		code = data_entry_add_field (data_entry, MAINT_CAT_CODE, "Code:", 
				8, INPUT_UPPER, 1,11,2,1);
		data_entry_add_field (data_entry, MAINT_CAT_DESC, "Description:",
				32, INPUT_NORMAL, 1,12,9,1);
		disc = data_entry_add_widget (data_entry, MAINT_CAT_DISC, 
				gtk_toggle_button_new_with_label ("Allow Discount"),
				2,13,1,1);
		gtk_signal_connect (GTK_OBJECT(disc), "toggled",
			(GtkSignalFunc) maint_cat_disc_toggle, NULL);

		gtk_widget_grab_focus (code);
	}

	if (!GTK_WIDGET_VISIBLE(maint_cat_data_entry))
		gtk_widget_show (GTK_WIDGET(maint_cat_data_entry));
	cat_list_populate (GTK_CLIST(cat_list));

}
