------------------------
-- Create tables

---------------
-- stores table

drop table stores;

create table stores (
	id				int2,
	store_name		text,
	address			text,
	suburb			text,
	postcode		int4,
	phone			text,
	manager			text,
	modem			text,
	update_hour		int4,
	update_minute	int4,
	is_branch		bool,
	next_custnum	int4,		-- next custnumber in sequence for this store
	next_ordernum	int4		-- next ordernumber in sequence for this store
	);

create unique index stores_id on stores (id);

---------------
-- User table

drop table users;

create table users (
	username	text,
	passwd		text
	);

create unique index user_name on users (username);

---------------
-- employees

drop table employee;

create table employee (
	emp_code		text,
	emp_name		text,
	address			text,
	suburb			text,
	phone			text,
	comments		text,
	rate			float8,
	tax_num			text
	);

create unique index emp_code on employee (emp_code);

---------------
-- Employee times

drop table emp_times;

create table emp_times (
	emp_code		text,
	store_id		int2,
	the_date		date,
	start_time		time,
	end_time		time,
	hours			int2,
	minutes			int2,
	rate			float8,
	pay				float8,
	paid			bool
	);

create index emp_time on emp_times (emp_code);

----------------
-- Employee pays

drop table emp_pay;

create table emp_pay (
	emp_code		text,
	date_from		date,
	date_to			date,
	total_hours		int2,
	total_mins		int2,
	rate			float8,
	bonus			float8,
	tax				float8,
	super			float8,
	cash			bool,
	cheque_num		text
	);

create index emp_pay_emp on emp_pay (emp_code);

---------------
-- Customer table

drop table customers;

create table customers (
	custnum			int4,
	custname		text,
	address			text,
	phone			text,
	suburb			text,
	ubd				text,
	comments		text,
	lastorder_num	int4,
	lastorder_date	abstime,
	bad_cust		bool,
	discount		int4   -- % discount on food items
	);

create unique index cust_num on customers (custnum);

---------------
-- Item Categories

drop table category;

create table category (
	code			char,
	description		text,
	allow_discount	bool
	);

create unique index category_code on category (code);

---------------
-- Items

drop table items;

create table items (
	category		char,
	code			text,
	description		text,
	price			float8,
	change_price	bool,			-- allow this price to be changed
	print_qty		bool			-- include the qty on the docket ?
	);

create unique index item_code on items (code);

---------------
-- Suburbs table

drop table suburbs;

create table suburbs (
	suburb			text,
	postcode		int4,
	min_order		float8,
	ubd				text,
	store_id		int2		-- default store for this suburb
	);

create unique index suburbs_sub on suburbs (suburb);

----------------
-- minium order prices depending on store


drop table suburb_min;

create table suburb_min (
	store_id	int2,
	suburb		text,
	min_order	float8
	);

----------------
-- Orders table

drop table orders;

create table orders (
	ordernum		int4,
	store_id		int2,		-- which store took this order
	is_pickup		bool,
	custnum			int4,
	pickup_name		text,
	pickup_time		text,
	order_date		abstime,
	total_price		float8,		-- de-normalised for speed
	comments		text		-- additional per-order comments
	);

create index orders_num on orders (ordernum);

create index orders_cust on orders (custnum);

---------------
-- Order Details

drop table ordet;

create table ordet (
	store_id		int2,
	ordernum		int4,
	seq_num			int4,
	item_code		text,
	description		text,
	qty				float8,
	price			float8
	);

create index ordet_order on ordet (ordernum);

---------------
-- Order changes

drop table ordhist;

create table ordhist (
	store_id		int2,
	ordernum		int4,
	seq_num			int4,
	item_code		text,
	description		text,
	qty				float8,
	price			float8,
	date			abstime
	);

create index ordhist_order on ordhist (ordernum);
