#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "data_entry.h"
#include <unistd.h>
#include <libpq-fe.h>
#include <gdk/gdkkeysyms.h>
#include "pizza_utils.h"
#include "select_screen.h"
#include "dockets.h"
#include "order_entry.h"

DataEntry *cust_details = NULL;

static GtkLabel *id, *lastorder_num, *lastorder_date;
static GtkInput *name,*address,*phone,*ubd,*discount;
static GtkEntry *suburb_entry;
static GtkCombo *suburb;
static GtkText *comments;
static GtkToggleButton *bad_cust;
static GtkCList *order_list;
static GtkPixmap *pix;

static int customer_number, is_new_cust;

/*----------------------------------------------------------------------------------------------------
::cust_maintenance functions
----------------------------------------------------------------------------------------------------*/

enum {
	CUSTDET_PIX,
	CUSTDET_ID,
	CUSTDET_DISC,
	CUSTDET_BAD,
	CUSTDET_NAME,
	CUSTDET_ADDRESS,
	CUSTDET_PHONE,
	CUSTDET_SUBURB,
	CUSTDET_UBD,
	CUSTDET_COMMENTS,
	CUSTDET_LASTORDER_NUM,
	CUSTDET_LASTORDER_DATE,
	CUSTDET_ORDER_LIST,

	CUSTDET_ACCEPT_BTN,
	CUSTDET_QUIT_BTN
};

static void cust_details_accept_callback (GtkWidget *widget, gpointer data)
{
	int sub_store, nTuples;
	char command[1024];
	char *ubd_code, *suburb_text;
	char *name_text, *phone_text, *address_text;
	PGresult *sres = NULL;

	// validate the entry fields first
	name_text = gtk_input_get_text (name);
	phone_text = gtk_input_get_text (phone);
	address_text = gtk_input_get_text (address);

	if (!strlen (address_text))
	{
		message_box ("You MUST enter an address for deliveries");
		gtk_widget_grab_focus (address);
		return;
	}

	if (!strlen (name_text) && !strlen (phone_text))
	{
		message_box ("You MUST enter either a name or a phone number");
		gtk_widget_grab_focus (name);
		return;
	}

	ubd_code = SQL(gtk_input_get_text (ubd));
	suburb_text = SQL(strupr(gtk_entry_get_text (suburb_entry)));
	if (is_new_cust)
	{
		// sort out the suburb
		sprintf (command, "select store_id from suburbs where suburb = '%s'",
			suburb_text);
		printf ("Command: '%s'\n", command);
		res = PQexec (conn, command);
		nTuples = PQntuples (res);
		if (nTuples)
		{
			sub_store = atoi (PQgetvalue (res,0,0));
			if (sub_store != current_store)
			{
				// found out which store then
				PQclear (res);
				sprintf (command, "select store_name,suburb,phone from stores where id = %d",
					sub_store);
				printf ("Command: '%s'\n", command);
				res = PQexec (conn, command);
				sprintf (command, "Please call : %s\nfor %s\nat %s for your order",
					PQgetvalue (res,0,2), 
					PQgetvalue (res,0,0), 
					PQgetvalue (res,0,1)); 

				message_box (command);
				PQclear (res);
				return;
			}
			PQclear (res);
		}
		else // no suburb
		{
			PQclear (res);
			sprintf (command, "insert into suburbs (suburb,ubd,store_id) "
							  "values ('%s','%s',%d)",
							  suburb_text,
							  ubd_code,
							  current_store);
			printf ("Command: '%s'\n", command);
			res = PQexec (conn, command);
			PQclear (res);
		}

		customer_number = get_new_customer_number ();

		// sort out the UBD code
		if (!strlen (ubd_code))
		{
			sprintf (command, "select ubd from suburbs where suburb = '%s'",
				suburb_text
				);
			printf ("Command: '%s'\n", command);
			sres = PQexec (conn, command);
			ubd_code = PQgetvalue (sres,0,0);
		}

		sprintf (command, "insert into customers (custnum,custname,address,phone,suburb,ubd,comments,"
			"bad_cust,discount) values (%d,'%s','%s','%s','%s','%s','%s','%c',%d)",
			customer_number,
			SQL(gtk_input_get_text (name)),
			SQL(gtk_input_get_text (address)),
			SQL(gtk_input_get_text (phone)),
			suburb_text,
			ubd_code,
			SQL(gtk_editable_get_chars (comments,0,-1)),
			bad_cust->active ? 't' : 'f',
			atoi(gtk_input_get_text (discount))
			);
		printf ("Command: '%s'\n", command);
		res = PQexec (conn, command);
		PQclear (res);

		if (sres) PQclear (sres);
	}
	else // update the customer
	{
		// sort out the suburb
		sprintf (command, "select store_id from suburbs where suburb = '%s'",
			suburb_text);
		printf ("Command: '%s'\n", command);
		res = PQexec (conn, command);
		nTuples = PQntuples (res);
		if (nTuples)
		{
			sub_store = atoi (PQgetvalue (res,0,0));
			if (sub_store != current_store)
			{
				// found out which store then
				PQclear (res);
				sprintf (command, "select store_name,suburb,phone from stores where id = %d",
					sub_store);
				printf ("Command: '%s'\n", command);
				res = PQexec (conn, command);
				sprintf (command, "Please call : %s\nfor %s\nat %s for your order",
					PQgetvalue (res,0,2), 
					PQgetvalue (res,0,0), 
					PQgetvalue (res,0,1)); 

				message_box (command);
				PQclear (res);
				return;
			}
			PQclear (res);
		}
		else // no suburb
		{
			PQclear (res);
			sprintf (command, "insert into suburbs (suburb,ubd,store_id) "
							  "values ('%s','%s',%d)",
							  suburb_text,
							  ubd_code,
							  current_store);
			printf ("Command: '%s'\n", command);
			res = PQexec (conn, command);
			PQclear (res);
		}

		// sort out the UBD code
		if (!strlen (ubd_code))
		{
			sprintf (command, "select ubd from suburbs where suburb = '%s'",
				suburb_text
				);
			printf ("Command: '%s'\n", command);
			sres = PQexec (conn, command);
			ubd_code = PQgetvalue (sres,0,0);
		}

		sprintf (command, "update customers set custname = '%s',"
			"address = '%s',phone = '%s',suburb = '%s',ubd = '%s',comments = '%s',"
			"bad_cust = '%c',discount = %d "
			"where custnum = %d",
			SQL(gtk_input_get_text (name)),
			SQL(gtk_input_get_text (address)),
			SQL(gtk_input_get_text (phone)),
			suburb_text,
			ubd_code,
			SQL(gtk_editable_get_chars (comments,0,-1)),
			bad_cust->active ? 't' : 'f',
			atoi(gtk_input_get_text (discount)),
			customer_number
			);
		printf ("Command: '%s'\n", command);
		res = PQexec (conn, command);
		PQclear (res);

		if (sres) PQclear (sres);
	}

	hide_docket_display ();
	gtk_widget_hide (GTK_WIDGET(cust_details));
	new_order (FALSE, customer_number);
}

static void cust_details_quit_callback (GtkWidget *widget, gpointer data)
{
	hide_docket_display ();
	gtk_widget_hide (GTK_WIDGET(cust_details));
	select_screen_redisplay ();
}

static void cust_ord_list_populate ()
{
	int i,nTuples;
	int j,nTuples2;
	char *ord_list_row[4];
	char order_buffer[32];
	char value_buffer[32];
	static char Buffer[4096];   // should be ample to hold and entire order !
	char command[255];
	PGresult *res2;
	int qty;

	int store_id, order_id;

	gtk_clist_freeze (order_list);
	gtk_clist_clear (order_list);

	sprintf (command, "select store_id,ordernum,order_date,total_price from orders where "
			"custnum = %d order by order_date DESC",
			customer_number);
	printf ("Command: '%s'\n", command);

	res = PQexec (conn,command);
	nTuples = PQntuples (res);
	for (i = 0; i < nTuples; i++)
	{
		store_id = atoi(PQgetvalue (res,i,0));
		order_id = atoi(PQgetvalue (res,i,1));
		sprintf (order_buffer, "%d-%d", store_id, order_id);
		ord_list_row[0] = order_buffer;
		ord_list_row[1] = nice_date (PQgetvalue (res,i,2));
		sprintf (value_buffer, "%1.02f", atof(PQgetvalue (res,i,3)));
		ord_list_row[3] = value_buffer;

		// get the description
		sprintf (command, "select qty,item_code from ordet where ordernum = %d and store_id = %d",
				order_id, store_id);
		printf ("Command: '%s'\n", command);
		res2 = PQexec (conn, command);
		nTuples2 = PQntuples (res2);
		*Buffer = 0;
		for (j = 0; j < nTuples2; j++)
		{
			if (j) strcat (Buffer, ", ");
			qty = atoi(PQgetvalue (res2, j, 0));
			if (qty > 1)
				sprintf (Buffer + strlen(Buffer), "%d ", qty);
			strcat (Buffer, PQgetvalue (res2, j, 1));
		}
		PQclear (res2);
		ord_list_row[2] = Buffer;
		gtk_clist_append (order_list, ord_list_row);
	}
	PQclear (res);

	gtk_clist_thaw (order_list);

	gtk_widget_grab_focus (name);
}

void refresh_cust_details ()
{
	cust_ord_list_populate ();
}

static void cust_det_ord_select (GtkWidget *widget, gint row, gint col, GdkEventButton *event, gpointer data)
{
	int store_id, order_id;
	char *text;
	char *ptr;

	gtk_clist_get_text (order_list, row, 0, &text);
	store_id = atoi (text);
	ptr = strchr (text, '-');
	order_id = atoi (ptr+1);

	display_docket (store_id, order_id);
}

static void cust_det_bad_toggled (GtkToggleButton *widget)
{
	if (widget->active)
	{
		reload_image (pix, "BadCust.jpg");
	}
	else
	{
		reload_image (pix, is_new_cust ? "NewCust.jpg" : "Customer.jpg");
	}
}

void do_cust_details (int old_cust_id, char *newname, char *newphone, int new_cust)
{
	DataEntry *data_entry;
	GtkAcceleratorTable *accel;
	GList *suburbs = NULL;
	GtkWidget *commentbox, *commentscrollbar;
	int i, nTuples;
	char command[1024];
	char *ptr;
	PGresult *cust_res;
	char *titles[] = {
		"Order", "Date", "Details", "Value"
	};

	is_new_cust = new_cust;
	customer_number = old_cust_id;
	if (is_new_cust) customer_number = 0;

	if (!cust_details)
	{
        order_list = GTK_CLIST(gtk_clist_new_with_titles (4, titles));
        gtk_clist_set_column_width (GTK_CLIST(order_list), 0, 60);                                      
		gtk_clist_set_column_width (GTK_CLIST(order_list), 1, 80);
		gtk_clist_set_column_width (GTK_CLIST(order_list), 2, 400);
        gtk_clist_set_column_width (GTK_CLIST(order_list), 3, 40);                                      
        gtk_clist_set_column_justification (GTK_CLIST (order_list), 3, GTK_JUSTIFY_RIGHT);

        gtk_widget_set_usize (GTK_WIDGET(order_list), 100, 150);
        gtk_signal_connect (GTK_OBJECT (order_list), "select_row",
            GTK_SIGNAL_FUNC(cust_det_ord_select), data_entry);

		// populate the suburbs array
    	cust_res = PQexec(conn, "select suburb from suburbs order by suburb");
		nTuples = PQntuples (cust_res);
		for (i = 0; i < nTuples; i++)
		{
			suburbs = g_list_append (suburbs, strdup(PQgetvalue(cust_res,i,0)));
		}
		PQclear (cust_res);

		cust_details = DATA_ENTRY(data_entry_new (new_cust ? "New Customer Details" : "Customer Details", 
				10, 40, 0,0));
		data_entry = cust_details;

		gtk_widget_realize (GTK_WIDGET(cust_details));

		accel = gtk_accelerator_table_new ();
		gtk_window_add_accelerator_table (GTK_WINDOW(data_entry), accel);

		data_entry_add_label_button (data_entry, CUSTDET_ACCEPT_BTN, "Accept", 
				GTK_SIGNAL_FUNC(cust_details_accept_callback), BUTTON_DEFAULT);
		data_entry_add_label_button (data_entry, CUSTDET_QUIT_BTN, "Quit", 
				GTK_SIGNAL_FUNC(cust_details_quit_callback), BUTTON_QUIT);

		// now add the fields

		pix = data_entry_add_image (data_entry, CUSTDET_PIX, new_cust ? "NewCust.jpg" : "Customer.jpg",
			4,7,2,2);
		data_entry_add_widget (data_entry, -1, GTK_WIDGET(gtk_label_new("ID:")), 0,6,1,1);
		id = GTK_LABEL(data_entry_add_widget (data_entry, CUSTDET_ID, GTK_WIDGET(gtk_label_new("")), 
			1,6,1,1));
		discount = GTK_INPUT(data_entry_add_field (data_entry, CUSTDET_DISC, "Discount % :", 
			3, INPUT_UPPER, 3,6,1,1));
		bad_cust = GTK_TOGGLE_BUTTON(data_entry_add_widget (data_entry, CUSTDET_BAD, 
			GTK_WIDGET(gtk_check_button_new_with_label ("Bad Customer")), 4,6,2,1));
		gtk_signal_connect (GTK_OBJECT(bad_cust), "toggled",
			GTK_SIGNAL_FUNC(cust_det_bad_toggled), data_entry);
		name = GTK_INPUT(data_entry_add_field (data_entry, CUSTDET_NAME, "Name:", 32, INPUT_UPPER,
			1,7,3,1));
		address = GTK_INPUT(data_entry_add_field (data_entry, CUSTDET_ADDRESS, "Address:", 32, INPUT_UPPER,
			1,8,3,1));
		phone = GTK_INPUT(data_entry_add_field (data_entry, CUSTDET_PHONE, "Phone:", 10, INPUT_UPPER,
			1,9,2,1));
		data_entry_add_widget (data_entry, -1, GTK_WIDGET(gtk_label_new ("Suburb:")), 0,10,1,1);

		suburb = GTK_COMBO(gtk_combo_new ());
    	suburb_entry = GTK_ENTRY((gtk_container_children (GTK_CONTAINER(suburb)))->data);
		gtk_combo_set_popdown_strings (GTK_COMBO(suburb), suburbs);
		data_entry_add_widget (data_entry, CUSTDET_SUBURB, GTK_WIDGET(suburb), 
			1,10,3,1);

		ubd = GTK_INPUT(data_entry_add_field (data_entry, CUSTDET_UBD, "UBD:", 4, INPUT_UPPER,
			5,10,1,1));

		// bit of code to add a comment entry area
		commentbox = gtk_hbox_new (FALSE,0);
		gtk_widget_show (GTK_WIDGET(commentbox));
		comments = GTK_TEXT(gtk_text_new (NULL, NULL));
		gtk_widget_set_usize (GTK_WIDGET(comments), 640, 60);
		gtk_text_set_word_wrap (GTK_TEXT(comments), TRUE);
		gtk_text_set_editable (GTK_TEXT(comments), TRUE);
		gtk_widget_show (GTK_WIDGET(comments));
		gtk_box_pack_start (GTK_BOX(commentbox), GTK_WIDGET(comments), FALSE, FALSE, 0);
		commentscrollbar = gtk_vscrollbar_new (comments->vadj);
		gtk_widget_show (commentscrollbar);
		gtk_box_pack_start (GTK_BOX(commentbox), commentscrollbar, FALSE, FALSE, 0);
		data_entry_add_widget (data_entry, -1, GTK_WIDGET(gtk_label_new ("Comments:")), 0,11,1,1);
		data_entry_add_widget (data_entry, CUSTDET_COMMENTS, GTK_WIDGET(commentbox), 1,11,7,1);

		data_entry_add_widget (data_entry, -1, GTK_WIDGET(gtk_label_new ("Last Order:")), 0, 12, 1,1);
		lastorder_num = GTK_LABEL(data_entry_add_widget (data_entry, CUSTDET_LASTORDER_NUM, 
			GTK_WIDGET(gtk_label_new ("")), 1,12,1,1));
		lastorder_date = GTK_LABEL(data_entry_add_widget (data_entry, CUSTDET_LASTORDER_DATE, 
			GTK_WIDGET(gtk_label_new ("")), 2,12,5,1));

		data_entry_add_widget (data_entry, CUSTDET_ORDER_LIST, GTK_WIDGET(order_list), 1,13,10,1);

	}

	if (!GTK_WIDGET_VISIBLE(cust_details))
		gtk_widget_show (GTK_WIDGET(cust_details));

	if (new_cust)
	{
		gtk_window_set_title (GTK_WINDOW(cust_details), "New Customer Details");
		// clear all the fields
		gtk_label_set (id, "");
		gtk_label_set (lastorder_num, "");
		gtk_label_set (lastorder_date, "");
		gtk_input_set_text (name, newname);
		gtk_input_set_text (address, "");
		gtk_input_set_text (phone, newphone);
		gtk_input_set_text (ubd, "");
		gtk_input_set_text (discount, "");
		gtk_entry_set_text (suburb_entry, "ADELAIDE");
		gtk_editable_delete_text (GTK_EDITABLE(comments), 0, -1);
		gtk_toggle_button_set_state (bad_cust, FALSE);
		gtk_clist_clear (order_list);
		reload_image (pix, "NewCust.jpg");
	}
	else
	{
		gtk_window_set_title (GTK_WINDOW(cust_details), "Customer Details");
		// populate all the fields
		sprintf (command, "select custnum,custname,address,phone,suburb,ubd,comments,"
				"lastorder_num,lastorder_date,bad_cust,discount from customers "
				"where custnum = %d",
				old_cust_id);
		printf ("Command: '%s'\n", command);

		cust_res = PQexec (conn, command);
		nTuples = PQntuples (cust_res);

		gtk_label_set (id, PQgetvalue(cust_res,0,0));
		gtk_label_set (lastorder_num, PQgetvalue (cust_res,0,7));
		gtk_label_set (lastorder_date, nice_date(PQgetvalue (cust_res,0,8)));
		gtk_input_set_text (name, PQgetvalue (cust_res,0,1));
		gtk_input_set_text (address, PQgetvalue (cust_res,0,2));
		gtk_input_set_text (phone, PQgetvalue (cust_res,0,3));
		gtk_entry_set_text (suburb_entry, PQgetvalue (cust_res,0,4));
		gtk_input_set_text (ubd, PQgetvalue (cust_res,0,5));
		gtk_input_set_text (discount, PQgetvalue (cust_res,0,10));
		gtk_editable_delete_text (GTK_EDITABLE(comments), 0, -1);
		ptr = PQgetvalue (cust_res,0,6);
		gtk_text_insert (comments, NULL, NULL,NULL,ptr,strlen(ptr));

		if (*PQgetvalue (cust_res,0,9) == 't')
		{
			gtk_toggle_button_set_state (bad_cust, TRUE);
			reload_image (pix, "BadCust.jpg");
		}
		else
		{
			gtk_toggle_button_set_state (bad_cust, FALSE);
			reload_image (pix, is_new_cust ? "NewCust.jpg" : "Customer.jpg");
		}

		PQclear (cust_res);
		cust_ord_list_populate ();

		// validate that they belong to this store ! - wonderous 3 way table join
		// and lightning fast it is too !

		sprintf (command, "select c.suburb,u.store_id,s.id,s.store_name,s.suburb,s.phone "
				"from customers c, suburbs u, stores s "
				"where c.custnum = %d and u.suburb = c.suburb and s.id = u.store_id",
				old_cust_id);
		printf ("Command: '%s'\n", command);
		cust_res = PQexec (conn, command);
		if (cust_res && PQntuples(cust_res))
		{
			i = atoi (PQgetvalue (cust_res,0,1));
			if (i != current_store)
			{
				// found out which store then
				sprintf (command, "Please call : %s\nfor %s\nat %s for your order",
					PQgetvalue (cust_res,0,5), 
					PQgetvalue (cust_res,0,3), 
					PQgetvalue (cust_res,0,4)); 

				message_box (command);
			}
			PQclear (cust_res);
		}
	}
	gtk_widget_grab_focus (GTK_WIDGET(name));
}
