#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "data_entry.h"
#include <unistd.h>
#include <libpq-fe.h>
#include <gdk/gdkkeysyms.h>
#include "pizza_utils.h"

DataEntry *maint_cust_data_entry = NULL;

static GtkText *comments;
static GtkToggleButton *search_bad, *search_disc;
static GtkCList *cust_list = NULL;
static GtkInput *search_text;

// TODO - add text search capability
// TODO - fix comments in and out

/*----------------------------------------------------------------------------------------------------
::cust_maintenance functions
----------------------------------------------------------------------------------------------------*/

/*----
	<sort order radio btns - ID, name, suburb, lastorder_date>
	<bad cust> <discount>
	<search name> [search]

	[list of customers      ]

	ID				discount
	name
	address
	phone
	suburb		ubd
	comments
	lastorder_num    lastorder_date
	bad_cust

	<ins> <delete> <update> <quit>

-----*/

enum {
	MAINT_CUST_SORT_ORDER,
	MAINT_CUST_SEARCH_BAD,
	MAINT_CUST_SEARCH_DISC,
	MAINT_CUST_SEARCH_TEXT,
	MAINT_CUST_SEARCH_BTN,
	MAINT_CUST_LIST,
	MAINT_CUST_ID,
	MAINT_CUST_DISC,
	MAINT_CUST_NAME,
	MAINT_CUST_ADDRESS,
	MAINT_CUST_PHONE,
	MAINT_CUST_SUBURB,
	MAINT_CUST_UBD,
	MAINT_CUST_COMMENTS,
	MAINT_CUST_LASTORDER_NUM,
	MAINT_CUST_LASTORDER_DATE,
	MAINT_CUST_BAD,

	MAINT_CUST_INS_BTN,
	MAINT_CUST_DEL_BTN,
	MAINT_CUST_MOD_BTN,
	MAINT_CUST_QUIT_BTN
};

static void cust_list_populate (GtkCList *cust_list);


static void maint_cust_search_clicked (GtkWidget *widget, gpointer data)
{
	cust_list_populate (cust_list);
}

// focus in on the code field - set default btn to update
static void maint_cust_name_focus_in (GtkWidget *widget, GdkEventFocus *event, gpointer data)
{
	DataEntry *data_entry;

	data_entry = (DataEntry *) data;

	gtk_widget_grab_default (data_entry_get_widget (data_entry, MAINT_CUST_MOD_BTN));
}

// focus in on the search item field - set default btn to search
static void maint_cust_search_focus_in (GtkWidget *widget, GdkEventFocus *event, gpointer data)
{
	DataEntry *data_entry;

	data_entry = (DataEntry *) data;

	gtk_widget_grab_default (data_entry_get_widget (data_entry, MAINT_CUST_SEARCH_BTN));
}

static void maint_cust_callback (GtkWidget *widget, gpointer data)
{
	DataEntry *data_entry;
	GtkInput *name, *addr,*phone,*ubd,*disc;
	GtkLabel *id, *lastorder_date, *lastorder_num;
	GtkWidget *suburb;
	GtkEntry *suburb_entry;
	GtkToggleButton *bad;
	GtkToggleButton *insert_btn;
	int btn_num;
	char command[4096];
	int customer_number;
	char *text;

	data_entry = (DataEntry *) data;
	btn_num = data_entry_get_widget_id (data_entry, widget);

	id = GTK_LABEL(data_entry_get_widget (data_entry, MAINT_CUST_ID));
	name = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_CUST_NAME));
	addr = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_CUST_ADDRESS));
	phone = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_CUST_PHONE));
	suburb = GTK_WIDGET(data_entry_get_widget (data_entry, MAINT_CUST_SUBURB));
	suburb_entry = GTK_ENTRY((gtk_container_children (GTK_CONTAINER(suburb)))->data);
	ubd = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_CUST_UBD));
	lastorder_num = GTK_LABEL(data_entry_get_widget (data_entry, MAINT_CUST_LASTORDER_NUM));
	lastorder_date = GTK_LABEL(data_entry_get_widget (data_entry, MAINT_CUST_LASTORDER_DATE));
	disc = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_CUST_DISC));
	bad = GTK_TOGGLE_BUTTON(data_entry_get_widget (data_entry, MAINT_CUST_BAD));

	insert_btn = GTK_TOGGLE_BUTTON(data_entry_get_widget (data_entry, MAINT_CUST_INS_BTN));

	switch (btn_num)
	{
		case MAINT_CUST_MOD_BTN:
			if (insert_btn->active)
			{
				// create a new customer number first
				customer_number = get_new_customer_number ();

				// now add a new customer
				sprintf (command, "insert into customers "
					"(custnum,custname,address,phone,suburb,ubd,comments,bad_cust,discount) "
					"values (%d,'%s','%s','%s','%s','%s','%s','%c',%d)",
					customer_number,
					SQL(gtk_input_get_text (name)),
					SQL(gtk_input_get_text (addr)),
					SQL(gtk_input_get_text (phone)),
					SQL(gtk_entry_get_text (suburb_entry)),
					SQL(gtk_input_get_text (ubd)),
					SQL(gtk_editable_get_chars(comments,0,-1)),
					bad->active ? 't' : 'f',
					atoi(gtk_input_get_text (disc))
					);
			}
			else // modify
			{
				gtk_label_get (id, &text);
				customer_number = atoi (text);

				sprintf (command, "update customers set "
					"custname = '%s',address = '%s',phone = '%s',suburb = '%s',ubd = '%s',"
					"comments = '%s',bad_cust = '%c',discount = %d "
					"where custnum = %d",
					SQL(gtk_input_get_text (name)),
					SQL(gtk_input_get_text (addr)),
					SQL(gtk_input_get_text (phone)),
					SQL(gtk_entry_get_text (suburb_entry)),
					SQL(gtk_input_get_text (ubd)),
					SQL(gtk_editable_get_chars(comments,0,-1)),
					bad->active ? 't' : 'f',
					atoi(gtk_input_get_text (disc)),
					customer_number
					);
			}
			printf ("Command = '%s'\n", command);
			res = PQexec (conn, command);
			PQclear (res);
			gtk_label_set (id, "");
			gtk_input_set_text (name, "");
			gtk_input_set_text (addr, "");
			gtk_input_set_text (phone, "");
			gtk_entry_set_text (suburb_entry, "");
			gtk_input_set_text (ubd, "");
			gtk_input_set_text (disc, "");
			gtk_editable_delete_text (comments, 0, -1);
			gtk_label_set (lastorder_num, "");
			gtk_label_set (lastorder_date, "");

			gtk_widget_grab_focus (GTK_WIDGET(cust_list));
			gtk_toggle_button_set_state (insert_btn, FALSE);
			cust_list_populate (cust_list);
			break;

		case MAINT_CUST_INS_BTN:
			gtk_widget_grab_focus (GTK_WIDGET(name));
			break;

		case MAINT_CUST_DEL_BTN:
			// delete the selected record
			gtk_label_get (id, &text);
			customer_number = atoi (text);
			sprintf (command, "delete from customers where custnum = %d", customer_number);
				
			printf ("Command = '%s'\n", command);
			res = PQexec (conn, command);
			PQclear (res);
			gtk_label_set (id, "");
			gtk_input_set_text (name, "");
			gtk_input_set_text (addr, "");
			gtk_input_set_text (phone, "");
			gtk_entry_set_text (suburb_entry, "");
			gtk_input_set_text (ubd, "");
			gtk_input_set_text (disc, "");
			gtk_editable_delete_text (comments, 0, -1);
			gtk_label_set (lastorder_num, "");
			gtk_label_set (lastorder_date, "");
			gtk_widget_grab_focus (GTK_WIDGET(cust_list));
			gtk_toggle_button_set_state (insert_btn, FALSE);
			cust_list_populate (cust_list);
			break;

		case MAINT_CUST_QUIT_BTN:
			gtk_widget_hide (GTK_WIDGET(data_entry));
			break;
	}
}

static void cust_list_populate (GtkCList *cust_list)
{
	int i, nTuples;
	char *cust_list_row[6];
	char command[4096];
	char *ptr, *orderby;
	char *blank = "";
	int do_bad, do_disc, do_search;
	char *search_for;
	char search_clause[1024];

	gtk_clist_freeze (cust_list);
	gtk_clist_clear (cust_list);

	do_bad = search_bad->active;
	do_disc = search_disc->active;

	// set orderby to depend on the radio btn
	orderby = data_entry_get_radio_value (maint_cust_data_entry, MAINT_CUST_SORT_ORDER);

	search_for = SQL(gtk_input_get_text (search_text));
	if (strlen (search_for))
	{
		do_search = 1;
		sprintf (search_clause, "(custname like '%c%s%c' OR address like '%c%s%c' OR "
				"phone like '%s%c' OR suburb like '%s%c') ",
				'%',search_for,'%',
				'%',search_for,'%',
				search_for,'%',
				search_for,'%');
	}
	else
		do_search = 0;

	if (do_bad && do_disc)
	{
		if (do_search)
		{
    		sprintf (command, "select custnum,custname,address,phone,suburb,lastorder_date "
				"from customers where (bad_cust = 't' AND discount > 0) AND %s"
				"order by %s", 
				search_clause, orderby);
		}
		else
		{
    		sprintf (command, "select custnum,custname,address,phone,suburb,lastorder_date "
				"from customers where bad_cust = 't' AND discount > 0 "
				"order by %s", orderby);
		}
	} else if (do_bad && !do_disc)
	{
		if (do_search)
		{
    		sprintf (command, "select custnum,custname,address,phone,suburb,lastorder_date "
				"from customers where bad_cust = 't' AND %s" 
				"order by %s", 
				search_clause, orderby);
		}
		else
		{
    		sprintf (command, "select custnum,custname,address,phone,suburb,lastorder_date "
				"from customers where bad_cust = 't' order by %s", orderby);
		}
	} else if (!do_bad && do_disc)
	{
		if (do_search)
		{
    		sprintf (command, "select custnum,custname,address,phone,suburb,lastorder_date "
				"from customers where discount > 0 AND %s"
				"order by %s", 
				search_clause, orderby);
		}
		else
		{
    		sprintf (command, "select custnum,custname,address,phone,suburb,lastorder_date "
				"from customers where discount > 0 order by %s", orderby);
		}
	}
	else
	{
		if (do_search)
		{
    		sprintf (command, "select custnum,custname,address,phone,suburb,lastorder_date "
				"from customers where %s order by %s", search_clause, orderby);
		}
		else
		{
    		sprintf (command, "select custnum,custname,address,phone,suburb,lastorder_date "
				"from customers order by %s", orderby);
		}
	}


	printf ("Command = '%s'\n", command);
    res = PQexec(conn, command);
    if (PQresultStatus(res) != PGRES_TUPLES_OK)
    {
        fprintf(stderr, "\nSelect didn't return tuples properly\n");
        PQclear(res);
        exit_nicely(conn);
    }

	nTuples = PQntuples (res);
	for (i = 0; i < nTuples; i++)
	{
		cust_list_row[0] = PQgetvalue (res, i, 0);
		cust_list_row[1] = PQgetvalue (res, i, 1);
		cust_list_row[2] = PQgetvalue (res, i, 2);
		cust_list_row[3] = PQgetvalue (res, i, 3);
		cust_list_row[4] = PQgetvalue (res, i, 4);
		cust_list_row[5] = nice_date (PQgetvalue (res, i, 5));
		gtk_clist_append (GTK_CLIST(cust_list), cust_list_row);
	}
	gtk_clist_thaw (cust_list);

	PQclear (res);
}

static void maint_cust_list_select (GtkWidget *widget, gint row, gint col, GdkEventButton *event, gpointer data)
{
	DataEntry *data_entry;
	GtkToggleButton *insert_btn;
	char command[256];
	gchar *text;
	GtkInput *name, *addr,*phone,*ubd,*disc;
	GtkLabel *id, *lastorder_date, *lastorder_num;
	char *ptr;
	GtkWidget *suburb;
	GtkEntry *suburb_entry;
	GtkToggleButton *bad;

	g_return_if_fail (widget);
	g_return_if_fail (data);
	data_entry = DATA_ENTRY (data);

	id = GTK_LABEL(data_entry_get_widget (data_entry, MAINT_CUST_ID));
	name = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_CUST_NAME));
	addr = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_CUST_ADDRESS));
	phone = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_CUST_PHONE));
	suburb = GTK_WIDGET(data_entry_get_widget (data_entry, MAINT_CUST_SUBURB));
	suburb_entry = GTK_ENTRY((gtk_container_children (GTK_CONTAINER(suburb)))->data);
	ubd = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_CUST_UBD));
	lastorder_num = GTK_LABEL(data_entry_get_widget (data_entry, MAINT_CUST_LASTORDER_NUM));
	lastorder_date = GTK_LABEL(data_entry_get_widget (data_entry, MAINT_CUST_LASTORDER_DATE));
	disc = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_CUST_DISC));
	bad = GTK_TOGGLE_BUTTON(data_entry_get_widget (data_entry, MAINT_CUST_BAD));

	insert_btn = GTK_TOGGLE_BUTTON(data_entry_get_widget (data_entry, MAINT_CUST_INS_BTN));

	gtk_clist_get_text (GTK_CLIST(widget), row, 0, &text);
	gtk_label_set (id, text);
	gtk_object_set_user_data (GTK_OBJECT(id), text);

	sprintf (command, "select custname,address,phone,suburb,ubd,comments,lastorder_num,"
					"lastorder_date,bad_cust,discount from customers where custnum = %d",
					atoi(text));

	res = PQexec (conn, command);

	gtk_input_set_text (name, PQgetvalue (res, 0, 0));
	gtk_input_set_text (addr, PQgetvalue (res, 0, 1));
	gtk_input_set_text (phone, PQgetvalue (res, 0, 2));
	gtk_entry_set_text (suburb_entry, PQgetvalue (res, 0, 3));
	gtk_input_set_text (ubd, PQgetvalue (res, 0, 4));
	ptr = PQgetvalue (res, 0, 5);
	gtk_editable_delete_text (comments, 0, -1);
	gtk_text_insert (GTK_TEXT(comments), NULL,NULL,NULL, ptr, strlen(ptr));
	gtk_label_set (lastorder_num, PQgetvalue (res, 0, 6));
	gtk_label_set (lastorder_date, nice_date (PQgetvalue (res, 0, 7)));

	if (*PQgetvalue (res, 0, 8) == 't')
		gtk_toggle_button_set_state (bad, TRUE);
	else
		gtk_toggle_button_set_state (bad, FALSE);
	gtk_input_set_text (disc, PQgetvalue (res, 0, 9));

	PQclear (res);

	gtk_widget_grab_focus (GTK_WIDGET(name));

	gtk_toggle_button_set_state (insert_btn, FALSE);
}

static void maint_cust_sort_toggled (GtkToggleButton *widget, gpointer data)
{
	DataEntry *data_entry;

	data_entry = (DataEntry *) data;

	//gtk_widget_grab_default (data_entry_get_widget (data_entry, MAINT_CUST_SEARCH_BTN));
	if (widget->active)
		cust_list_populate (data_entry_get_widget (data_entry, MAINT_CUST_LIST));
}

static void maint_cust_search_toggled (GtkToggleButton *widget, gpointer data)
{
	DataEntry *data_entry;

	data_entry = (DataEntry *) data;

	cust_list_populate (data_entry_get_widget (data_entry, MAINT_CUST_LIST));
}

void maint_custs (GtkWidget *widget, gpointer data)
{
	DataEntry *data_entry;
	static GtkWidget *name,*suburb;
	static GtkWidget *sort_order, *search_btn;
	
	GtkAcceleratorTable *accel;
	GtkWidget *delbtn;
	GtkBox *commentbox;
	GtkWidget *commentscrollbar;
	int nTuples,i;
	GList *suburbs = NULL;
	GtkWidget *radio_btn;
	static char *titles[] = 
	{
		"ID","Name","Address","Phone","Suburb","Last Date"
	};

	if (!maint_cust_data_entry)
	{
		// populate the suburbs array
    	res = PQexec(conn, "select suburb from suburbs order by suburb");
		nTuples = PQntuples (res);
		for (i = 0; i < nTuples; i++)
		{
			suburbs = g_list_append (suburbs, strdup(PQgetvalue(res,i,0)));
		}
		PQclear (res);

		maint_cust_data_entry = DATA_ENTRY(data_entry_new ("Customers", 10, 40, 0,0));
		data_entry = maint_cust_data_entry;

		accel = gtk_accelerator_table_new ();
		gtk_window_add_accelerator_table (GTK_WINDOW(data_entry), accel);

		cust_list = gtk_clist_new_with_titles (6, titles);
		// TODO - set the correct colun widths
		gtk_clist_set_column_width (GTK_CLIST(cust_list), 0, 40);
		gtk_clist_set_column_width (GTK_CLIST(cust_list), 1, 120);
		gtk_clist_set_column_width (GTK_CLIST(cust_list), 2, 150);
		gtk_clist_set_column_width (GTK_CLIST(cust_list), 3, 90);
		gtk_clist_set_column_width (GTK_CLIST(cust_list), 4, 110);
		gtk_clist_set_column_width (GTK_CLIST(cust_list), 5, 160);
		gtk_widget_set_usize (GTK_WIDGET(cust_list), 670, 140);
		gtk_signal_connect (GTK_OBJECT (cust_list), "select_row", 
			(GtkSignalFunc) maint_cust_list_select, data_entry);

		data_entry_add_widget (data_entry, MAINT_CUST_LIST, GTK_WIDGET(cust_list), 0,5,10,1);

		data_entry_add_label_button (data_entry, MAINT_CUST_INS_BTN, "Insert", 
				GTK_SIGNAL_FUNC(maint_cust_callback), BUTTON_NORMAL | BUTTON_TOGGLE);

		delbtn = data_entry_add_label_button (data_entry, MAINT_CUST_DEL_BTN, "Delete", 
				GTK_SIGNAL_FUNC(maint_cust_callback), BUTTON_NORMAL);
		gtk_accelerator_table_install (accel, GTK_OBJECT(delbtn), "clicked", '', 0);

		data_entry_add_label_button (data_entry, MAINT_CUST_MOD_BTN, "Update", 
				GTK_SIGNAL_FUNC(maint_cust_callback), BUTTON_DEFAULT);
		data_entry_add_label_button (data_entry, MAINT_CUST_QUIT_BTN, "Quit", 
				GTK_SIGNAL_FUNC(maint_cust_callback), BUTTON_QUIT);

		// now add the fields

		data_entry_add_widget (data_entry, -1, GTK_WIDGET(gtk_label_new ("Sort By:")), 0,0,1,1);
		sort_order = data_entry_add_radio_hbox (data_entry, MAINT_CUST_SORT_ORDER, 1,0,3,1);
		radio_btn = data_entry_add_to_radio (data_entry, sort_order, "custnum", "ID");
			gtk_signal_connect (GTK_OBJECT(radio_btn), "toggled",
				GTK_SIGNAL_FUNC (maint_cust_sort_toggled), data_entry);
		radio_btn = data_entry_add_to_radio (data_entry, sort_order, "custname", "Name");
			gtk_signal_connect (GTK_OBJECT(radio_btn), "toggled",
				GTK_SIGNAL_FUNC (maint_cust_sort_toggled), data_entry);
		radio_btn = data_entry_add_to_radio (data_entry, sort_order, "suburb", "Suburb");
			gtk_signal_connect (GTK_OBJECT(radio_btn), "toggled",
				GTK_SIGNAL_FUNC (maint_cust_sort_toggled), data_entry);
		radio_btn = data_entry_add_to_radio (data_entry, sort_order, "phone", "Phone");
			gtk_signal_connect (GTK_OBJECT(radio_btn), "toggled",
				GTK_SIGNAL_FUNC (maint_cust_sort_toggled), data_entry);
		radio_btn = data_entry_add_to_radio (data_entry, sort_order, "lastorder_date DESC", "Last Order");
			gtk_signal_connect (GTK_OBJECT(radio_btn), "toggled",
				GTK_SIGNAL_FUNC (maint_cust_sort_toggled), data_entry);

		search_bad = data_entry_add_widget (data_entry, MAINT_CUST_SEARCH_BAD, 
			GTK_WIDGET(gtk_toggle_button_new_with_label ("Bad Customers")), 4,0,2,1);

		gtk_signal_connect (GTK_OBJECT(search_bad), "toggled",
			GTK_SIGNAL_FUNC (maint_cust_search_toggled), data_entry);

		search_disc = data_entry_add_widget (data_entry, MAINT_CUST_SEARCH_DISC,
			GTK_WIDGET(gtk_toggle_button_new_with_label ("Discounts")), 6,0,1,1);

		gtk_signal_connect (GTK_OBJECT(search_disc), "toggled",
			GTK_SIGNAL_FUNC (maint_cust_search_toggled), data_entry);

		// now add the search stuff

		search_text = data_entry_add_field (data_entry, MAINT_CUST_SEARCH_TEXT, "Search For:", 
			40, INPUT_UPPER, 1,2,5,1);
			gtk_signal_connect (GTK_OBJECT(search_text), "focus_in_event",
				(GtkSignalFunc) maint_cust_search_focus_in, data_entry);
		search_btn = data_entry_add_widget (data_entry, MAINT_CUST_SEARCH_BTN, 
			GTK_WIDGET(gtk_button_new_with_label ("Search")), 6, 2,1,1);
		GTK_WIDGET_SET_FLAGS (search_btn, GTK_CAN_DEFAULT);
			gtk_signal_connect (GTK_OBJECT(search_btn), "clicked",
				GTK_SIGNAL_FUNC (maint_cust_search_clicked), data_entry);

		data_entry_add_widget (data_entry, -1, GTK_WIDGET(gtk_label_new("ID:")), 0,6,1,1);
		data_entry_add_widget (data_entry, MAINT_CUST_ID, GTK_WIDGET(gtk_label_new("")), 1,6,1,1);
		data_entry_add_field (data_entry, MAINT_CUST_DISC, "Discount % :", 3, INPUT_UPPER,
			3,6,1,1);
		data_entry_add_widget (data_entry, MAINT_CUST_BAD, 
			GTK_WIDGET(gtk_check_button_new_with_label ("Bad Customer")), 4,6,2,1);
		name = data_entry_add_field (data_entry, MAINT_CUST_NAME, "Name:", 32, INPUT_UPPER,
			1,7,3,1);
			gtk_signal_connect (GTK_OBJECT(name), "focus_in_event",
				(GtkSignalFunc) maint_cust_name_focus_in, data_entry);
		data_entry_add_field (data_entry, MAINT_CUST_ADDRESS, "Address:", 32, INPUT_UPPER,
			1,8,3,1);
		data_entry_add_field (data_entry, MAINT_CUST_PHONE, "Phone:", 10, INPUT_UPPER,
			1,9,2,1);
		data_entry_add_widget (data_entry, -1, GTK_WIDGET(gtk_label_new ("Suburb:")), 0,10,1,1);
		suburb = gtk_combo_new ();
		gtk_combo_set_popdown_strings (GTK_COMBO(suburb), suburbs);
		data_entry_add_widget (data_entry, MAINT_CUST_SUBURB, GTK_WIDGET(suburb), 
			1,10,3,1);

		data_entry_add_field (data_entry, MAINT_CUST_UBD, "UBD:", 4, INPUT_UPPER,
			5,10,1,1);

		// bit of code to add a comment entry area
		commentbox = gtk_hbox_new (FALSE,0);
		gtk_widget_show (GTK_WIDGET(commentbox));
		comments = gtk_text_new (NULL, NULL);
		gtk_widget_set_usize (GTK_WIDGET(comments), 640, 60);
		gtk_text_set_word_wrap (GTK_TEXT(comments), TRUE);
		gtk_text_set_editable (GTK_TEXT(comments), TRUE);
		gtk_widget_show (GTK_WIDGET(comments));
		gtk_box_pack_start (commentbox, GTK_WIDGET(comments), FALSE, FALSE, 0);
		commentscrollbar = gtk_vscrollbar_new (GTK_TEXT(comments)->vadj);
		gtk_widget_show (commentscrollbar);
		gtk_box_pack_start (commentbox, commentscrollbar, FALSE, FALSE, 0);
		data_entry_add_widget (data_entry, -1, GTK_WIDGET(gtk_label_new ("Comments:")), 0,11,1,1);
		data_entry_add_widget (data_entry, MAINT_CUST_COMMENTS, GTK_WIDGET(commentbox), 1,11,7,1);

		data_entry_add_widget (data_entry, -1, GTK_WIDGET(gtk_label_new ("Last Order:")), 0, 12, 1,1);
		data_entry_add_widget (data_entry, MAINT_CUST_LASTORDER_NUM, 
			GTK_WIDGET(gtk_label_new ("")), 1,12,1,1);
		data_entry_add_widget (data_entry, MAINT_CUST_LASTORDER_DATE, 
			GTK_WIDGET(gtk_label_new ("")), 2,12,5,1);

		gtk_widget_grab_focus (name);
	}

	if (!GTK_WIDGET_VISIBLE(maint_cust_data_entry))
		gtk_widget_show (GTK_WIDGET(maint_cust_data_entry));
	cust_list_populate (GTK_CLIST(cust_list));

}
