/* GTK - The GIMP Toolkit
 * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef __GTK_INPUT_H__
#define __GTK_INPUT_H__


#include <gdk/gdk.h>
#include <gtk/gtkeditable.h>


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#define GTK_INPUT(obj)          GTK_CHECK_CAST (obj, gtk_input_get_type (), GtkInput)
#define GTK_INPUT_CLASS(klass)  GTK_CHECK_CLASS_CAST (klass, gtk_input_get_type (), GtkInputClass)
#define GTK_IS_INPUT(obj)       GTK_CHECK_TYPE (obj, gtk_input_get_type ())


typedef struct _GtkInput       GtkInput;
typedef struct _GtkInputClass  GtkInputClass;

#define INPUT_NORMAL 0
#define INPUT_UPPER  1
#define INPUT_LOWER  2

struct _GtkInput
{
  GtkEditable editable;

  GdkWindow *text_area;
  GdkPixmap *backing_pixmap;
  GdkCursor *cursor;
  gchar *text;
  gchar *view_text;

  guint16 text_size;
  guint16 text_length;
  guint16 text_max_length;
  gint16  scroll_offset;
  guint   visible : 1;
  guint32 timer;
  gchar	  echo_char;
  gint16  flag;
};

struct _GtkInputClass
{
  GtkEditableClass parent_class;

  void (* activate)     (GtkInput    *input);
};


guint      gtk_input_get_type       (void);
GtkWidget* gtk_input_new            (void);
GtkWidget* gtk_input_new_with_flag  (guint16 max, gchar echochar, gint16 flag);
GtkWidget* gtk_input_new_with_max_length (guint16   max);
void       gtk_input_set_text       (GtkInput      *input,
				     const gchar   *text);
void       gtk_input_append_text    (GtkInput      *input,
				     const gchar   *text);
void       gtk_input_prepend_text   (GtkInput      *input,
				     const gchar   *text);
void       gtk_input_set_position   (GtkInput      *input,
				     gint           position);
gchar*     gtk_input_get_text       (GtkInput      *input);
gchar*     gtk_input_get_full_text  (GtkInput      *input);
gint       gtk_input_get_int       (GtkInput      *input);
double     gtk_input_get_double    (GtkInput      *input);
void       gtk_input_select_region  (GtkInput      *input,
				     gint           start,
				     gint           end);
void       gtk_input_set_visibility (GtkInput      *input,
				     gboolean       visible);
void       gtk_input_set_editable   (GtkInput      *input,
				     gboolean       editable);

/* If input->text is already > max it's up to you to change it */
void       gtk_input_set_max_length (GtkInput      *input,
                                     guint16        max);


#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* __GTK_INPUT_H__ */
