#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "data_entry.h"
#include <unistd.h>
#include <libpq-fe.h>
#include <gdk/gdkkeysyms.h>
#include "pizza_utils.h"
#include "select_cust.h"
#include "order_entry.h"

DataEntry *select_screen = NULL;

static GtkInput *phone, *name, *order;
static GtkWidget *search_cat;
static GtkLabel *orders_today, *total_today;
static GtkCombo *how_many, *store_select;
static GtkCList *order_list;
static GtkToggleButton *refresh_btn;

static GtkEntry *how_many_entry, *store_select_entry;

static int ok_to_populate = 0;

static int num_orders_today = 0;
static double total_value_today = 0.0;
static char default_store_string[255];			// the default value whenever we are shown

#define USE_TEST_DATE

static void select_populate ();

/*----------------------------------------------------------------------------------------------------
::select screen
----------------------------------------------------------------------------------------------------*/
enum {
	SELECT_PIX,
	SELECT_PHONE,
	SELECT_NAME,
	SELECT_ORDER,
	SELECT_SEARCH_CAT,
	SELECT_STORE,

	SELECT_LIST,

	SELECT_ORDERS_TODAY,
	SELECT_TOTAL_TODAY,
	SELECT_HOWMANY,

	SELECT_REFRESH_BTN,
	SELECT_REPORT_BTN,
	SELECT_ACCEPT_BTN,
	SELECT_QUIT_BTN
};

static int get_selected_store ()
{
	return atoi (gtk_entry_get_text (store_select_entry));
}

static char *get_pickup_clause ()
{
	static char Buffer[32];
	int the_type;

	the_type = atoi (data_entry_get_radio_value (select_screen, SELECT_SEARCH_CAT));

	switch (the_type)
	{
		case 2: // pickups only
			strcpy (Buffer, "is_pickup = 't'");
			break;

		case 3: // deliveries only
			strcpy (Buffer, "is_pickup = 'f'");
			break;

		case 1:	// all
		default:
			*Buffer = 0;
			break;
	}

	return Buffer;
}

static char *get_date_clause ()
{
	static char Buffer[1024];
	static char today[20],yesterday[20],tomorrow[20];
	int current_hour;
	char *use_day, *next_day;
	char *ptr;

	ptr = gtk_entry_get_text (how_many_entry);

	// get current date from database
	res = PQexec (conn, "select 'today'::date,'yesterday'::date,'tomorrow'::date,'now'::time");
	strcpy (today, PQgetvalue(res,0,0));
	strcpy (yesterday, PQgetvalue(res,0,1));
	strcpy (tomorrow, PQgetvalue(res,0,2));
	current_hour = atoi(PQgetvalue(res,0,3));
	if (current_hour < 7)	// in the wee hours of the morning - use previous day
	{
		use_day = yesterday;
		next_day = today;
	}
	else
	{
		use_day = today;
		next_day = tomorrow;
	}

#ifdef USE_TEST_DATE
	use_day = "03-07-1998";
	next_day = "03-08-1998";
#endif

	if (!strcmp (ptr, "Today"))
	{
		sprintf (Buffer, "(order_date >= '%s 08:00' AND order_date < '%s 08:00')",
			use_day, next_day);
	}
	else if (!strcmp (ptr, "Yesterday"))
	{
		sprintf (Buffer, "(order_date >= ('%s 08:00'::abstime - '1 day'::reltime) AND "
				"order_date < ('%s 08:00'::abstime - '1 day'::reltime))",
			use_day, next_day);
	}
	else if (!strcmp (ptr, "Whole Week"))
	{
		sprintf (Buffer, "(order_date >= ('%s 08:00'::abstime - '1 week'::reltime) AND "
				"order_date < '%s 08:00')",
			use_day, next_day);
	}
	else if (!strcmp (ptr, "Show All"))
	{
		*Buffer = 0;
	}

	return Buffer;
}

static void select_search_cat_toggled (GtkToggleButton *btn, gpointer data)
{
	if (btn->active)
		select_populate();
}

static void select_populate ()
{
	static char details[1024];
	static char command[1024];

	int nTuples,i;
	char *order_list_row[5];
	char order_number[32];
	char store_string[32];
	char value[32];
	int is_pickup;
	int use_store_id;
	PGresult *cust_res;
	char *date_clause, *pickup_clause;
	char *where_bit;
	int max_cust_lookup = 300;
	int do_and = 0;

	if (!ok_to_populate) return;

	gtk_clist_freeze (order_list);
	gtk_clist_clear (order_list);

	use_store_id = get_selected_store ();

	if (use_store_id)
		sprintf (store_string, "store_id = %d", use_store_id);
	else
		*store_string = 0;

	date_clause = get_date_clause ();
	pickup_clause = get_pickup_clause ();

	where_bit = "";

	if (strlen(store_string) || strlen(date_clause) || strlen(pickup_clause))
		where_bit = "where";

	sprintf (command, "select store_id,ordernum,order_date,is_pickup,custnum,"
					"pickup_name,pickup_time,total_price "
					"from orders %s ", where_bit);

	if (strlen (store_string))
	{
		strcat (command, store_string);
		do_and = 1;
	}

	if (strlen (date_clause))
	{
		if (do_and) strcat (command, " AND ");
		do_and = 1;
		strcat (command, date_clause);
	}

	if (strlen (pickup_clause))
	{
		if (do_and) strcat (command, " AND ");
		strcat (command, pickup_clause);
	}

	strcat (command, " order by order_date DESC");
	printf ("Command: '%s'\n", command);
	res = PQexec (conn, command);

	num_orders_today = 0;
	total_value_today = 0.0;

	nTuples = PQntuples (res);
	for (i = 0; i < nTuples; i++)
	{
		num_orders_today++;
		total_value_today += atof(PQgetvalue (res,i,7));

		sprintf (order_number, "%d-%d", atoi(PQgetvalue (res,i,0)), atoi(PQgetvalue (res,i,1)));
		order_list_row[0] = order_number;
		order_list_row[1] = nice_date(PQgetvalue (res,i,2));
		is_pickup = *PQgetvalue (res,i,3) == 't';
		if (is_pickup)
		{
			order_list_row[2] = "P";
			sprintf (details, "%s at %s", trim(PQgetvalue (res,i,5)),trim(PQgetvalue (res,i,6)));
			order_list_row[3] = details;
		}
		else
		{
			order_list_row[2] = "";

			if (--max_cust_lookup > 0)
			{
				sprintf (command, "select phone,custname,address,suburb from customers where custnum = %d",
					atoi(PQgetvalue (res,i,4)));
				cust_res = PQexec (conn, command);
				if (PQntuples(cust_res) > 0)
				{
					sprintf (details, "%s %s %s %s",
						PQgetvalue (cust_res,0,0),	
						PQgetvalue (cust_res,0,1),	
						PQgetvalue (cust_res,0,2),	
						PQgetvalue (cust_res,0,3));	
				}
				else
					*details = 0;
				PQclear (cust_res);
			}
			else
			{
				sprintf (details, "Customer %d", atoi(PQgetvalue (res,i,4)));
			}

			order_list_row[3] = details;
		}
		sprintf (value, "%01.2f", atof(PQgetvalue (res,i,7)));
		order_list_row[4] = value;
		gtk_clist_append (order_list, order_list_row);
	}
	PQclear (res);

	gtk_clist_thaw (order_list);

	sprintf (details, "%d", num_orders_today);
	gtk_label_set (orders_today, details);
	sprintf (details, "%.2f", total_value_today);
	gtk_label_set (total_today, details);

	gtk_widget_grab_focus (GTK_WIDGET(phone));
}

static void select_report_callback (GtkWidget *widget, gpointer data)
{
}

static void select_ord_list_select (GtkWidget *widget, gint row, gint col, 
	GdkEventButton *event, gpointer data)                    
{
    int store_id, order_id, cust_id;
    char *text;
    char *ptr;
	char command[255];

	printf ("btn %d\n", event->button);

    gtk_clist_get_text (order_list, row, 0, &text);
    store_id = atoi (text);
    ptr = strchr (text, '-');
    order_id = atoi (ptr+1);
	gtk_clist_get_text (order_list, row, 2, &text);

	if (event->button == 1)
	{
		gtk_widget_hide (select_screen);
		old_order (store_id, order_id, *text == 'P');
	}
	else
	{
		if (*text != 'P')
		{
			sprintf (command, "select custnum from orders where ordernum = %d and store_id = %d",
				order_id, store_id);
			printf ("Command: '%s'\n", command);
			res = PQexec (conn, command);
			cust_id = atoi (PQgetvalue (res,0,0));
			PQclear (res);
			if (cust_id)
			{
				gtk_widget_hide (select_screen);
				do_cust_details (cust_id,NULL,NULL,FALSE);
			}
		}
	}
}

static void select_accept_callback (GtkWidget *widget, gpointer data)
{
	char *phone_ptr, *name_ptr, *order_ptr;
	int the_order_store;
	int the_order;
	static char command[255];
	char *ptr;
	int i, nTuples;

	phone_ptr = gtk_input_get_text (phone);
	name_ptr = gtk_input_get_text (name);
	order_ptr = gtk_input_get_text (order);
	ptr = strchr (order_ptr, '-');

	if (ptr)
	{
		the_order_store = atoi (order_ptr);
		the_order = atoi (ptr+1);
	}
	else
	{
		the_order_store = current_store;
		the_order = atoi (order_ptr);
	}

	if (strlen (order_ptr))
	{
		printf ("selected store %d order %d\n", the_order_store, the_order);
	}

	if (the_order)	// get the order
	{
		sprintf (command, "select * from orders where store_id = %d and ordernum = %d",
					the_order_store, the_order);
		res = PQexec (conn, command);
		if (PQntuples (res) != 1)
		{
			sprintf (command, "ERROR: No matching order %d-%d", the_order_store, the_order);
			message_box (command);
		}
		else
		{
			sprintf (command, "OK: Found order %d-%d", the_order_store, the_order);
			message_box (command);
		}
		PQclear (res);
		gtk_input_set_text (phone, "");
		gtk_input_set_text (name, "");
		gtk_input_set_text (order, "");
		return;
	}
	else if (strlen(phone_ptr) && strlen(name_ptr))
	{
		sprintf (command, "select custnum,custname,address,phone,suburb,lastorder_date "
			"from customers where "
			"phone like '%s%c' AND "
			"(custname like '%c%s%c' or address like '%c%s%c' or comments like '%c%s%c') "
			"order by custnames",
			phone_ptr, '%',
			'%',name_ptr,'%',
			'%',name_ptr,'%',
			'%',name_ptr,'%');
	}
	else if (strlen(phone_ptr))
	{
		sprintf (command, "select custnum,custname,address,phone,suburb,lastorder_date "
			"from customers where "
			"phone like '%s%c' "
			"order by phone",
			phone_ptr, '%');
	}
	else if (strlen(name_ptr))
	{
		sprintf (command, "select custnum,custname,address,phone,suburb,lastorder_date "
			"from customers where "
			"(custname like '%c%s%c' or address like '%c%s%c' or comments like '%c%s%c') "
			"order by custname",
			'%',name_ptr,'%',
			'%',name_ptr,'%',
			'%',name_ptr,'%');
	}
	else // new pickup order
	{
		gtk_widget_hide (select_screen);
		new_order (TRUE,0);
		return;
	}

	printf ("Command = '%s'\n", command);
	res = PQexec (conn, command);
	nTuples = PQntuples (res);


	if (nTuples == 1)
	{
		i = atoi(PQgetvalue (res,0,0));
		PQclear (res);
		do_cust_details (i,NULL,NULL,FALSE);
		gtk_widget_hide (select_screen);
	}
	else if (nTuples < 1)
	{
		PQclear (res);
		do_cust_details (0, name_ptr, phone_ptr, TRUE);
		gtk_widget_hide (select_screen);
	}
	else 
	{
		do_select_cust (res);
		gtk_widget_hide (select_screen);
		PQclear (res);
	}

	gtk_widget_grab_focus (GTK_WIDGET(phone));
}

static void select_quit_callback (GtkWidget *widget, gpointer data)
{
	DataEntry *data_entry;

	data_entry = (DataEntry *) data;
	gtk_widget_show (GTK_WIDGET(login_screen));
	gtk_widget_hide (GTK_WIDGET(data_entry));
}

void do_select_screen ()
{
	DataEntry *data_entry;
	GtkWidget *window, *radio_btn;
	int i,nTuples;
	char store_string[255];	
	GList *stores = NULL;
	char *titles[] =
	{
		"Order", "Date", "P/D", "Details", "Value"
	};
	GList *strings = NULL;
	static char *how_many_strings[] =
	{
		"Today", "Yesterday", "Whole Week", "Show All", 
		NULL
	};

	ok_to_populate = 0;
	if (!select_screen)
	{
		data_entry = DATA_ENTRY(data_entry_new ("Main Selection Screen", 20, 20, 0,0));
		window = (GtkWidget *) data_entry;
		gtk_widget_realize (window);

		order_list = GTK_CLIST(gtk_clist_new_with_titles (5, titles));
		gtk_clist_set_column_width (GTK_CLIST(order_list), 0, 70);
		gtk_clist_set_column_width (GTK_CLIST(order_list), 1, 120);
		gtk_clist_set_column_width (GTK_CLIST(order_list), 2, 20);
		gtk_clist_set_column_width (GTK_CLIST(order_list), 3, 400);
		gtk_clist_set_column_width (GTK_CLIST(order_list), 4, 40);
		gtk_clist_set_column_justification (GTK_CLIST (order_list), 4, GTK_JUSTIFY_RIGHT);

		gtk_widget_set_usize (GTK_WIDGET(order_list), 750, 300);
		gtk_signal_connect (GTK_OBJECT (order_list), "select_row", 
			GTK_SIGNAL_FUNC(select_ord_list_select), data_entry);

		data_entry_add_widget (data_entry, SELECT_LIST, GTK_WIDGET(order_list), 0,4,10,1);

		refresh_btn = data_entry_add_label_button (data_entry, SELECT_REFRESH_BTN, "Refresh", 
				GTK_SIGNAL_FUNC(select_populate), BUTTON_TOGGLE);
		gtk_toggle_button_set_state (refresh_btn, true);
		data_entry_add_label_button (data_entry, SELECT_REPORT_BTN, "Report", 
				GTK_SIGNAL_FUNC(select_report_callback), BUTTON_NORMAL);
		data_entry_add_label_button (data_entry, SELECT_ACCEPT_BTN, "Accept", 
				GTK_SIGNAL_FUNC(select_accept_callback), BUTTON_DEFAULT);
		data_entry_add_label_button (data_entry, SELECT_QUIT_BTN, "Quit", 
				GTK_SIGNAL_FUNC(select_quit_callback), BUTTON_QUIT);

		// add the fields

		data_entry_add_image (data_entry, SELECT_PIX, "Select.jpg",
			4,0,1,1);

		phone = GTK_INPUT(data_entry_add_field (data_entry, SELECT_PHONE, "Phone:", 20, 
			INPUT_UPPER, 1,0,3,1));
		name = GTK_INPUT(data_entry_add_field (data_entry, SELECT_NAME, "Name:", 20, 
			INPUT_UPPER, 1,1,3,1));
		order = GTK_INPUT(data_entry_add_field (data_entry, SELECT_ORDER, "Order:", 10, 
			INPUT_UPPER, 1,2,1,1));

		// create and populate the radiobutii 
		search_cat = data_entry_add_radio_hbox (data_entry, SELECT_SEARCH_CAT, 4,1,1,1);
		radio_btn = data_entry_add_to_radio (data_entry, search_cat, "1", "All");
			gtk_signal_connect (GTK_OBJECT (radio_btn), "toggled", 
				GTK_SIGNAL_FUNC (select_search_cat_toggled), NULL);
		radio_btn = data_entry_add_to_radio (data_entry, search_cat, "2", "Pickup");
			gtk_signal_connect (GTK_OBJECT (radio_btn), "toggled", 
				GTK_SIGNAL_FUNC (select_search_cat_toggled), NULL);
		radio_btn = data_entry_add_to_radio (data_entry, search_cat, "3", "Deliv");
			gtk_signal_connect (GTK_OBJECT (radio_btn), "toggled", 
				GTK_SIGNAL_FUNC (select_search_cat_toggled), NULL);

		// store selection combo - default to current store
		sprintf (store_string ,"select id,store_name from stores where id = %d", current_store);
    	res = PQexec(conn, store_string);
		sprintf (default_store_string, "%d %s", atoi(PQgetvalue (res,0,0)), PQgetvalue (res,0,1));
		stores = g_list_append (stores, default_store_string);
		PQclear (res);

    	res = PQexec(conn, "select id,store_name from stores order by id");
		nTuples = PQntuples (res);
		for (i = 0; i < nTuples; i++)
		{
			if (atoi(PQgetvalue (res,i,0)) != current_store)
			{
				sprintf (store_string, "%d %s", atoi(PQgetvalue (res,i,0)), PQgetvalue (res,i,1));
				stores = g_list_append (stores, strdup(store_string));
			}
		}
		stores = g_list_append (stores, strdup("All Stores ..."));
		PQclear (res);
		store_select = GTK_COMBO(gtk_combo_new ());
		gtk_combo_set_popdown_strings (GTK_COMBO(store_select), stores);
		data_entry_add_widget (data_entry, -1, gtk_label_new ("Select:"), 3,2,1,1);
		data_entry_add_widget (data_entry, SELECT_STORE, GTK_WIDGET(store_select), 
			4,2,1,1);
		store_select_entry = GTK_ENTRY((gtk_container_children (GTK_CONTAINER(store_select)))->data);
		gtk_signal_connect (GTK_OBJECT(store_select_entry),"changed",
			GTK_SIGNAL_FUNC(select_populate),NULL);

		data_entry_add_widget (data_entry, -1, gtk_label_new ("Number of Orders:"), 0,15,1,1);
		orders_today = GTK_LABEL(data_entry_add_widget (data_entry, SELECT_ORDERS_TODAY,
			gtk_label_new (""), 1,15,1,1));
		data_entry_add_widget (data_entry, -1, gtk_label_new ("Total Orders:"), 0,16,1,1);
		total_today = GTK_LABEL(data_entry_add_widget (data_entry, SELECT_TOTAL_TODAY,
			gtk_label_new (""), 1,16,1,1));

		// add the howmany combo
		data_entry_add_widget (data_entry, -1, gtk_label_new ("Show Which Orders:"), 3,15,1,1);
		how_many = GTK_COMBO(data_entry_add_widget (data_entry, SELECT_HOWMANY,
			gtk_combo_new (), 4,15,2,1));

		i = 0;
		while (how_many_strings[i])
			strings = g_list_append (strings, how_many_strings[i++]);

		gtk_combo_set_popdown_strings (how_many, strings);
		how_many_entry = GTK_ENTRY((gtk_container_children (GTK_CONTAINER(how_many)))->data);
		gtk_signal_connect (GTK_OBJECT(how_many_entry),"changed",
			GTK_SIGNAL_FUNC(select_populate),NULL);

		select_screen = data_entry;
		gtk_widget_grab_focus (phone);
	}


	if (!GTK_WIDGET_VISIBLE (select_screen))
		gtk_widget_show (GTK_WIDGET(select_screen));

	// re-default the category to all
	data_entry_set_radio_value (select_screen, SELECT_SEARCH_CAT, "1");

	// re-default the store select to the current store
	gtk_entry_set_text (store_select_entry, default_store_string);

	// re-default the how many thing to today
	gtk_entry_set_text (how_many_entry, "Today");

	// clear out the entry fields
	gtk_input_set_text (phone, "");
	gtk_input_set_text (name, "");
	gtk_input_set_text (order, "");

	ok_to_populate = 1;
	select_populate ();
}

void select_screen_redisplay ()
{
	ok_to_populate = 0;
	gtk_widget_show (select_screen);
	data_entry_set_radio_value (select_screen, SELECT_SEARCH_CAT, "1");
	gtk_entry_set_text (store_select_entry, default_store_string);
	gtk_entry_set_text (how_many_entry, "Today");
	gtk_input_set_text (phone, "");
	gtk_input_set_text (name, "");
	gtk_input_set_text (order, "");
	ok_to_populate = 1;

	if (refresh_btn->active)
		select_populate ();
	gtk_widget_grab_focus (GTK_WIDGET(phone));
}
