#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "data_entry.h"
#include <unistd.h>
#include <libpq-fe.h>
#include <gdk/gdkkeysyms.h>
#include "pizza_utils.h"

DataEntry *maint_store_data_entry = NULL;

/*----------------------------------------------------------------------------------------------------
::store_maintenance functions
----------------------------------------------------------------------------------------------------*/

/*------
	[list of stores   ]

	fields [] ....

	<ins> <del> <update> <quit>
---------*/

enum {
	MAINT_STORE_LIST,
	MAINT_STORE_ID,
	MAINT_STORE_BRANCH,
	MAINT_STORE_NAME,
	MAINT_STORE_ADDR,
	MAINT_STORE_SUBURB,
	MAINT_STORE_PCODE,
	MAINT_STORE_PHONE,
	MAINT_STORE_MGR,
	MAINT_STORE_MODEM,
	MAINT_STORE_HOUR,
	MAINT_STORE_MIN,
	MAINT_STORE_NEXT_CUST,
	MAINT_STORE_NEXT_ORD,

	MAINT_STORE_INS_BTN,
	MAINT_STORE_DEL_BTN,
	MAINT_STORE_MOD_BTN,
	MAINT_STORE_QUIT_BTN
};

static void store_list_populate (GtkCList *store_list);

static void maint_store_callback (GtkWidget *widget, gpointer data)
{
	DataEntry *data_entry;
	GtkInput *id, *name, *addr, *suburb, *pcode, *phone, *mgr, *modem, *hour, *min, 
			 *next_cust, *next_ord;
	GtkCheckButton *branch;
	GtkToggleButton *insert_btn;
	GtkCList *store_list;
	int btn_num;
	char command[4096];

	data_entry = (DataEntry *) data;
	btn_num = data_entry_get_widget_id (data_entry, widget);

	store_list = GTK_CLIST(data_entry_get_widget (data_entry, MAINT_STORE_LIST));
	id = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_STORE_ID));
	name = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_STORE_NAME));
	addr = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_STORE_ADDR));
	suburb = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_STORE_SUBURB));
	pcode = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_STORE_PCODE));
	phone = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_STORE_PHONE));
	mgr = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_STORE_MGR));
	modem = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_STORE_MODEM));
	hour = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_STORE_HOUR));
	min = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_STORE_MIN));
	branch = GTK_CHECK_BUTTON(data_entry_get_widget (data_entry, MAINT_STORE_BRANCH));
	next_cust = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_STORE_NEXT_CUST));
	next_ord = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_STORE_NEXT_ORD));
	insert_btn = GTK_TOGGLE_BUTTON(data_entry_get_widget (data_entry, MAINT_STORE_INS_BTN));

	switch (btn_num)
	{
		case MAINT_STORE_MOD_BTN:
			if (insert_btn->active)
			{
				sprintf (command, "insert into stores values (%d,'%s','%s','%s',%d,'%s','%s','%s',%d,%d,"
					"'%c',%d,%d)",
					gtk_input_get_int (id),
					gtk_input_get_text (name),
					gtk_input_get_text (addr),
					gtk_input_get_text (suburb),
					gtk_input_get_int (pcode),
					gtk_input_get_text (phone),
					gtk_input_get_text (mgr),
					gtk_input_get_text (modem),
					gtk_input_get_int (hour),
					gtk_input_get_int (min),
					(GTK_TOGGLE_BUTTON(branch))->active ? 't' : 'f',
					gtk_input_get_int (next_cust),
					gtk_input_get_int (next_ord));

			}
			else // modify
			{
				sprintf (command, "update stores set id = %d, store_name = '%s', address = '%s', "
						"suburb = '%s', phone = '%s', postcode = %d, manager = '%s', modem = '%s', "
						"update_hour = %d, update_minute = %d, next_custnum = %d, "
						"next_ordernum = %d, is_branch = '%c' where id = %s",
					gtk_input_get_int (id),
					gtk_input_get_text (name),
					gtk_input_get_text (addr),
					gtk_input_get_text (suburb),
					gtk_input_get_text (phone),
					gtk_input_get_int (pcode),
					gtk_input_get_text (mgr),
					gtk_input_get_text (modem),
					gtk_input_get_int (hour),
					gtk_input_get_int (min),
					gtk_input_get_int (next_cust),
					gtk_input_get_int (next_ord),
					(GTK_TOGGLE_BUTTON(branch))->active ? 't' : 'f',
					(char *) gtk_object_get_user_data (GTK_OBJECT(id)));
			}
			//printf ("Command = '%s'\n", command);
			res = PQexec (conn, command);
			PQclear (res);
			gtk_input_set_text (id, "");
			gtk_input_set_text (name, "");
			gtk_input_set_text (addr, "");
			gtk_input_set_text (suburb, "");
			gtk_input_set_text (phone, "");
			gtk_input_set_text (pcode, "");
			gtk_input_set_text (mgr, "");
			gtk_input_set_text (modem, "");
			gtk_input_set_text (hour, "");
			gtk_input_set_text (min, "");
			gtk_input_set_text (next_cust, "");
			gtk_input_set_text (next_ord, "");
			gtk_widget_grab_focus (GTK_WIDGET(store_list));
			gtk_toggle_button_set_state (insert_btn, FALSE);
			store_list_populate (store_list);
			gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(branch), FALSE);
			break;

		case MAINT_STORE_INS_BTN:
			gtk_widget_grab_focus (GTK_WIDGET(id));
			break;

		case MAINT_STORE_DEL_BTN:
			// delete the selected record
			sprintf (command, "delete from stores where id = %d",
				gtk_input_get_int (id));
			//printf ("Command = '%s'\n", command);
			res = PQexec (conn, command);
			PQclear (res);
			store_list_populate (store_list);
			gtk_input_set_text (id, "");
			gtk_input_set_text (name, "");
			gtk_input_set_text (addr, "");
			gtk_input_set_text (suburb, "");
			gtk_input_set_text (phone, "");
			gtk_input_set_text (pcode, "");
			gtk_input_set_text (mgr, "");
			gtk_input_set_text (modem, "");
			gtk_input_set_text (hour, "");
			gtk_input_set_text (min, "");
			gtk_input_set_text (next_cust, "");
			gtk_input_set_text (next_ord, "");
			gtk_widget_grab_focus (GTK_WIDGET(store_list));
			gtk_toggle_button_set_state (insert_btn, FALSE);
			gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(branch), FALSE);
			break;

		case MAINT_STORE_QUIT_BTN:
			gtk_widget_hide (GTK_WIDGET(data_entry));
			break;
	}
}

static void store_list_populate (GtkCList *store_list)
{
	int i, nTuples;
	char *store_list_row[6];

	gtk_clist_freeze (store_list);
	gtk_clist_clear (store_list);
    //printf ("select id,store_name,address,suburb,phone,manager from stores order by id\n");
    res = PQexec (conn, "select id,store_name,address,suburb,phone,manager from stores order by id\n");
    //printf ("select custnum,custname,address,suburb,phone,ubd from customers order by custnum\n");
    //res = PQexec (conn, "select custnum,custname,address,suburb,phone,ubd from customers order by custnum");
    if (PQresultStatus(res) != PGRES_TUPLES_OK)
    {
        fprintf(stderr, "\nSelect didn't return tuples properly\n");
        PQclear(res);
        exit_nicely(conn);
    }

	nTuples = PQntuples (res);
	for (i = 0; i < nTuples; i++)
	{
		store_list_row[0] = PQgetvalue (res, i, 0);
		store_list_row[1] = PQgetvalue (res, i, 1);
		store_list_row[2] = PQgetvalue (res, i, 2);
		store_list_row[3] = PQgetvalue (res, i, 3);
		store_list_row[4] = PQgetvalue (res, i, 4);
		store_list_row[5] = PQgetvalue (res, i, 5);
		gtk_clist_append (GTK_CLIST(store_list), store_list_row);
	}
	gtk_clist_thaw (store_list);
	PQclear (res);
}

static void maint_store_list_select (GtkWidget *widget, gint row, gint col, GdkEventButton *event, gpointer data)
{
	DataEntry *data_entry;
	GtkInput *id, *input_field;
	GtkToggleButton *insert_btn;
	gchar *text;
	char command[255];
	int nFields, i,j;
	GtkCheckButton *branch;
	char *sqlvalue;

	g_return_if_fail (widget);
	g_return_if_fail (data);
	data_entry = DATA_ENTRY (data);

	id = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_STORE_ID));
	insert_btn = GTK_TOGGLE_BUTTON(data_entry_get_widget (data_entry, MAINT_STORE_INS_BTN));

	// get cell 1 of the selected row in the table
	gtk_clist_get_text (GTK_CLIST(widget), row, 0, &text);
	gtk_input_set_text (id, text);
	gtk_object_set_user_data (GTK_OBJECT(id), text);

	// lookup the database

	sprintf (command, "select is_branch,store_name,address,suburb,postcode,phone,manager,"
					  "modem,update_hour,update_minute,next_custnum,next_ordernum "
					  "from stores where id = %s", text);
	//printf ("%s\n", command);
	res = PQexec (conn, command);
	nFields = PQnfields (res);

	// populate all fields
	branch = GTK_CHECK_BUTTON(data_entry_get_widget (data_entry, MAINT_STORE_BRANCH));
	sqlvalue = PQgetvalue (res, 0, 0);
	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(branch), (*sqlvalue == 't'));

	j = 1;
	for (i = MAINT_STORE_NAME; i <= MAINT_STORE_NEXT_ORD; i++)
	{
		input_field = GTK_INPUT(data_entry_get_widget (data_entry, i));
		sqlvalue = PQgetvalue (res, 0, j++);  // sorry about having J++ in my code !
		gtk_input_set_text (input_field, sqlvalue);
	}
	PQclear (res);
	gtk_widget_grab_focus (GTK_WIDGET(id));

	gtk_toggle_button_set_state (insert_btn, FALSE);
}

void maint_stores (GtkWidget *widget, gpointer data)
{
	DataEntry *data_entry;
	static GtkWidget *store_list, *id;
	static char *titles[] = 
	{
		"ID", "Name", "Address", "Suburb", "Phone", "Manager"
	};

	if (!maint_store_data_entry)
	{
		maint_store_data_entry = DATA_ENTRY(data_entry_new ("Stores", 10, 80, 0,0));
		data_entry = maint_store_data_entry;

		store_list = gtk_clist_new_with_titles (6, titles);
		gtk_clist_set_column_width (GTK_CLIST(store_list), 0, 30);
		gtk_clist_set_column_width (GTK_CLIST(store_list), 1, 120);
		gtk_clist_set_column_width (GTK_CLIST(store_list), 2, 150);
		gtk_clist_set_column_width (GTK_CLIST(store_list), 3, 120);
		gtk_clist_set_column_width (GTK_CLIST(store_list), 4, 120);
		gtk_clist_set_column_width (GTK_CLIST(store_list), 5, 120);
		gtk_widget_set_usize (GTK_WIDGET(store_list), 750, 150);
		gtk_signal_connect (GTK_OBJECT (store_list), "select_row", 
			(GtkSignalFunc) maint_store_list_select, data_entry);

		data_entry_add_widget (data_entry, MAINT_STORE_LIST, store_list, 0,0,10,1);

		data_entry_add_label_button (data_entry, MAINT_STORE_INS_BTN, "Insert", 
				GTK_SIGNAL_FUNC(maint_store_callback), BUTTON_NORMAL | BUTTON_TOGGLE);
		data_entry_add_label_button (data_entry, MAINT_STORE_DEL_BTN, "Delete", 
				GTK_SIGNAL_FUNC(maint_store_callback), BUTTON_NORMAL);
		data_entry_add_label_button (data_entry, MAINT_STORE_MOD_BTN, "Update", 
				GTK_SIGNAL_FUNC(maint_store_callback), BUTTON_DEFAULT);
		data_entry_add_label_button (data_entry, MAINT_STORE_QUIT_BTN, "Quit", 
				GTK_SIGNAL_FUNC(maint_store_callback), BUTTON_QUIT);


		// now add the fields
		id = data_entry_add_field (data_entry, MAINT_STORE_ID, "ID:", 
				2, INPUT_UPPER, 1,1,1,1);
		//data_entry_add_widget (data_entry, -1, gtk_label_new ("Branch Store"),
				//2,1,1,1);
		data_entry_add_widget (data_entry, MAINT_STORE_BRANCH, 
				gtk_check_button_new_with_label ("Branch Store"),
				3,1,1,1);
		data_entry_add_field (data_entry, MAINT_STORE_NAME, "Name:",
				32, INPUT_NORMAL, 1,2,5,1);
		data_entry_add_field (data_entry, MAINT_STORE_ADDR, "Address:",
				32, INPUT_NORMAL, 1,3,5,1);
		data_entry_add_field (data_entry, MAINT_STORE_SUBURB, "Suburb:",
				25, INPUT_NORMAL, 1,4,3,1);
		data_entry_add_field (data_entry, MAINT_STORE_PCODE, "PCode:",
				4, INPUT_NORMAL, 5,4,1,1);
		data_entry_add_field (data_entry, MAINT_STORE_PHONE, "Phone:",
				12, INPUT_NORMAL, 1,5,2,1);
		data_entry_add_field (data_entry, MAINT_STORE_MGR, "Manager:",
				25, INPUT_NORMAL, 4,5,2,1);
		data_entry_add_field (data_entry, MAINT_STORE_MODEM, "Modem:",
				12, INPUT_NORMAL, 1,6,1,1);
		data_entry_add_field (data_entry, MAINT_STORE_HOUR, "Update Hour:",
				2, INPUT_NORMAL, 3,6,1,1);
		data_entry_add_field (data_entry, MAINT_STORE_MIN, NULL,
				2, INPUT_NORMAL, 4,6,1,1);
		data_entry_add_field (data_entry, MAINT_STORE_NEXT_CUST, "Next Cust:",
				6, INPUT_NORMAL, 1,8,1,1);
		data_entry_add_field (data_entry, MAINT_STORE_NEXT_ORD, "Next Order:",
				6, INPUT_NORMAL, 3,8,1,1);

		gtk_widget_grab_focus (id);
	}

	if (!GTK_WIDGET_VISIBLE(maint_store_data_entry))
		gtk_widget_show (GTK_WIDGET(maint_store_data_entry));
	store_list_populate (GTK_CLIST(store_list));
}
