#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "data_entry.h"
#include <unistd.h>
#include <libpq-fe.h>
#include <gdk/gdkkeysyms.h>
#include "pizza_utils.h"

DataEntry *maint_suburb_data_entry = NULL;

/*----------------------------------------------------------------------------------------------------
::suburb_maintenance functions
----------------------------------------------------------------------------------------------------*/

/*----
	<search cats>
	<search name>

	[list of suburbs      ]

	suburb [  ]  min order [   ]
	postcode [    ] ubd [   ]
	store   [combo]

	<ins> <delete> <update> <quit>

-----*/

enum {
	MAINT_SUBURB_LIST,
	MAINT_SUBURB_SUBURB,
	MAINT_SUBURB_POSTCODE,
	MAINT_SUBURB_MINORDER,
	MAINT_SUBURB_UBD,
	MAINT_SUBURB_STORE,

	MAINT_SUBURB_INS_BTN,
	MAINT_SUBURB_DEL_BTN,
	MAINT_SUBURB_MOD_BTN,
	MAINT_SUBURB_QUIT_BTN
};

static void suburb_list_populate (GtkCList *suburb_list);

static void maint_suburb_callback (GtkWidget *widget, gpointer data)
{
	DataEntry *data_entry;
	GtkInput *suburb, *minorder, *postcode, *ubd;
	GtkWidget *store;
	GtkToggleButton *insert_btn;
	GtkCList *suburb_list;
	int btn_num;
	char command[255];
	GtkEntry *store_entry;
	int store_number;

	data_entry = (DataEntry *) data;
	btn_num = data_entry_get_widget_id (data_entry, widget);

	suburb_list = GTK_CLIST(data_entry_get_widget (data_entry, MAINT_SUBURB_LIST));
	suburb = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_SUBURB_SUBURB));
	minorder = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_SUBURB_MINORDER));
	postcode = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_SUBURB_POSTCODE));
	ubd = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_SUBURB_UBD));
	store = GTK_WIDGET(data_entry_get_widget (data_entry, MAINT_SUBURB_STORE));
	store_entry = GTK_ENTRY((gtk_container_children (GTK_CONTAINER(store)))->data);
	store_number = atoi(gtk_entry_get_text(store_entry));
	insert_btn = GTK_TOGGLE_BUTTON(data_entry_get_widget (data_entry, MAINT_SUBURB_INS_BTN));

	switch (btn_num)
	{
		case MAINT_SUBURB_MOD_BTN:
			if (insert_btn->active)
			{
				sprintf (command, "insert into suburbs values ('%s',%d,%f,'%s',%d)",
					gtk_input_get_text (suburb),
					atoi(gtk_input_get_text (postcode)),
					atof(gtk_input_get_text (minorder)),
					gtk_input_get_text (ubd),
					store_number);
			}
			else // modify
			{
				sprintf (command, "update suburbs set suburb = '%s',postcode = %d,"
						"min_order = %f,ubd = '%s',store_id = %d where suburb = '%s'",
					gtk_input_get_text (suburb),
					atoi(gtk_input_get_text (postcode)),
					atof(gtk_input_get_text (minorder)),
					gtk_input_get_text (ubd),
					store_number,
					(char *) gtk_object_get_user_data (GTK_OBJECT(suburb)));
			}
			printf ("Command = '%s'\n", command);
			res = PQexec (conn, command);
			PQclear (res);
			gtk_input_set_text (suburb, "");
			gtk_input_set_text (minorder, "");
			gtk_input_set_text (postcode, "");
			gtk_input_set_text (ubd, "");
			gtk_entry_set_text (store_entry, "");
			gtk_widget_grab_focus (GTK_WIDGET(suburb_list));
			gtk_toggle_button_set_state (insert_btn, FALSE);
			suburb_list_populate (suburb_list);
			break;

		case MAINT_SUBURB_INS_BTN:
			gtk_widget_grab_focus (GTK_WIDGET(suburb));
			break;

		case MAINT_SUBURB_DEL_BTN:
			// delete the selected record
			sprintf (command, "delete from suburbs where suburb = '%s'", 
				gtk_input_get_text (suburb));
			//printf ("Command = '%s'\n", command);
			res = PQexec (conn, command);
			PQclear (res);
			gtk_input_set_text (suburb, "");
			gtk_input_set_text (minorder, "");
			gtk_input_set_text (postcode, "");
			gtk_input_set_text (ubd, "");
			gtk_entry_set_text (store_entry, "");
			gtk_widget_grab_focus (GTK_WIDGET(suburb_list));
			gtk_toggle_button_set_state (insert_btn, FALSE);
			suburb_list_populate (suburb_list);
			break;

		case MAINT_SUBURB_QUIT_BTN:
			gtk_widget_hide (GTK_WIDGET(data_entry));
			break;
	}
}

static void suburb_list_populate (GtkCList *suburb_list)
{
	int i, nTuples;
	char *suburb_list_row[5];
	char command[256];

	gtk_clist_freeze (suburb_list);
	gtk_clist_clear (suburb_list);
    sprintf (command, "select s.suburb,s.postcode,s.min_order,s.ubd,s.store_id,t.store_name "
				"from suburbs s,stores t where t.id = s.store_id order by s.suburb");
	//printf ("Command = '%s'\n", command);
    res = PQexec(conn, command);
    if (PQresultStatus(res) != PGRES_TUPLES_OK)
    {
        fprintf(stderr, "\nSelect didn't return tuples properly\n");
        PQclear(res);
        exit_nicely(conn);
    }

	nTuples = PQntuples (res);
	for (i = 0; i < nTuples; i++)
	{
		suburb_list_row[0] = PQgetvalue (res, i, 0);
		suburb_list_row[1] = PQgetvalue (res, i, 1);
		suburb_list_row[2] = PQgetvalue (res, i, 2);
		suburb_list_row[3] = PQgetvalue (res, i, 3);
		sprintf (command, "%d %s", atoi(PQgetvalue (res, i, 4)), PQgetvalue (res, i, 5));
		suburb_list_row[4] = command;
		gtk_clist_append (GTK_CLIST(suburb_list), suburb_list_row);
	}
	gtk_clist_thaw (suburb_list);
	PQclear (res);
}

static void maint_suburb_list_select (GtkWidget *widget, gint row, gint col, GdkEventButton *event, gpointer data)
{
	DataEntry *data_entry;
	GtkInput *suburb,*postcode,*min_order,*ubd;
	GtkToggleButton *insert_btn;
	char command[256];
	gchar *text;
	GtkWidget *store_id;
	GtkEntry *store_entry;

	g_return_if_fail (widget);
	g_return_if_fail (data);
	data_entry = DATA_ENTRY (data);

	suburb = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_SUBURB_SUBURB));
	postcode = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_SUBURB_POSTCODE));
	min_order = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_SUBURB_MINORDER));
	ubd = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_SUBURB_UBD));
	store_id = GTK_WIDGET(data_entry_get_widget (data_entry, MAINT_SUBURB_STORE));
	store_entry = GTK_ENTRY((gtk_container_children (GTK_CONTAINER(store_id)))->data);
	insert_btn = GTK_TOGGLE_BUTTON(data_entry_get_widget (data_entry, MAINT_SUBURB_INS_BTN));

	gtk_clist_get_text (GTK_CLIST(widget), row, 0, &text);
	gtk_input_set_text (suburb, text);
	gtk_object_set_user_data (GTK_OBJECT(suburb), text);

	sprintf (command, "select s.postcode,s.min_order,s.ubd,s.store_id,t.store_name "
					"from suburbs s,stores t where s.suburb = '%s' and t.id = s.store_id", text);

	res = PQexec (conn, command);
	gtk_input_set_text (postcode, PQgetvalue (res, 0, 0));
	gtk_input_set_text (min_order, PQgetvalue (res, 0, 1));
	gtk_input_set_text (ubd, PQgetvalue (res, 0, 2));
	sprintf (command, "%d %s", atoi (PQgetvalue (res, 0, 3)), PQgetvalue (res, 0, 4));
	gtk_entry_set_text (store_entry, command);
	PQclear (res);

	gtk_widget_grab_focus (GTK_WIDGET(suburb));

	gtk_toggle_button_set_state (insert_btn, FALSE);
}

void maint_suburbs (GtkWidget *widget, gpointer data)
{
	DataEntry *data_entry;
	static GtkWidget *suburb_list, *suburb,*postcode,*min_order,*ubd,*store_id;
	GtkAcceleratorTable *accel;
	GtkWidget *delbtn;
	int nTuples,i;
	GList *stores = NULL;
	static char *titles[] = 
	{
		"Suburb","Postcode","Min Order","UBD","Store"
	};
	char Buffer[255];

	if (!maint_suburb_data_entry)
	{
		// populate the array of valid stores
    	res = PQexec(conn, "select id, store_name from stores order by id");
		nTuples = PQntuples (res);
		for (i = 0; i < nTuples; i++)
		{
			sprintf (Buffer, "%d %s", atoi(PQgetvalue (res, i, 0)), PQgetvalue (res, i, 1));
			stores = g_list_append (stores, strdup(Buffer));
		}
		PQclear (res);

		maint_suburb_data_entry = DATA_ENTRY(data_entry_new ("Suburbs", 10, 140, 0,0));
		data_entry = maint_suburb_data_entry;

		accel = gtk_accelerator_table_new ();
		gtk_window_add_accelerator_table (GTK_WINDOW(data_entry), accel);

		suburb_list = gtk_clist_new_with_titles (5, titles);
		gtk_clist_set_column_width (GTK_CLIST(suburb_list), 0, 160);
		gtk_clist_set_column_width (GTK_CLIST(suburb_list), 1, 80);
		gtk_clist_set_column_width (GTK_CLIST(suburb_list), 2, 80);
		gtk_clist_set_column_width (GTK_CLIST(suburb_list), 3, 50);
		gtk_clist_set_column_width (GTK_CLIST(suburb_list), 4, 150);
		gtk_widget_set_usize (GTK_WIDGET(suburb_list), 660, 200);
		gtk_signal_connect (GTK_OBJECT (suburb_list), "select_row", 
			(GtkSignalFunc) maint_suburb_list_select, data_entry);

		data_entry_add_widget (data_entry, MAINT_SUBURB_LIST, suburb_list, 0,0,10,1);

		data_entry_add_label_button (data_entry, MAINT_SUBURB_INS_BTN, "Insert", 
				GTK_SIGNAL_FUNC(maint_suburb_callback), BUTTON_NORMAL | BUTTON_TOGGLE);

		delbtn = data_entry_add_label_button (data_entry, MAINT_SUBURB_DEL_BTN, "Delete", 
				GTK_SIGNAL_FUNC(maint_suburb_callback), BUTTON_NORMAL);
		gtk_accelerator_table_install (accel, GTK_OBJECT(delbtn), "clicked", '', 0);

		data_entry_add_label_button (data_entry, MAINT_SUBURB_MOD_BTN, "Update", 
				GTK_SIGNAL_FUNC(maint_suburb_callback), BUTTON_DEFAULT);
		data_entry_add_label_button (data_entry, MAINT_SUBURB_QUIT_BTN, "Quit", 
				GTK_SIGNAL_FUNC(maint_suburb_callback), BUTTON_QUIT);

		// now add the fields
		suburb = data_entry_add_field (data_entry, MAINT_SUBURB_SUBURB, "Suburb:", 
				20, INPUT_UPPER, 1,10,5,1);
		postcode = data_entry_add_field (data_entry, MAINT_SUBURB_POSTCODE, "Postcode:",
				4, INPUT_NORMAL, 7,10,1,1);
		min_order = data_entry_add_field (data_entry, MAINT_SUBURB_MINORDER, "Min Order:",
				4, INPUT_NORMAL, 1,11,1,1);
		ubd = data_entry_add_field (data_entry, MAINT_SUBURB_UBD, "UBD:",
				6, INPUT_UPPER, 3,11,1,1);
		data_entry_add_widget (data_entry, -1, gtk_label_new ("Main Store:"), 0,12,1,1);
		store_id = gtk_combo_new ();
		gtk_combo_set_popdown_strings (GTK_COMBO(store_id), stores);
		store_id = data_entry_add_widget (data_entry, MAINT_SUBURB_STORE, store_id, 1,12,3,1);

		gtk_widget_grab_focus (suburb);
	}

	if (!GTK_WIDGET_VISIBLE(maint_suburb_data_entry))
		gtk_widget_show (GTK_WIDGET(maint_suburb_data_entry));
	suburb_list_populate (GTK_CLIST(suburb_list));

}
