#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "data_entry.h"
#include <unistd.h>
#include <libpq-fe.h>
#include <gdk/gdkkeysyms.h>
#include "pizza_utils.h"

DataEntry *maint_user_data_entry = NULL;

/*----------------------------------------------------------------------------------------------------
::user_maintenance functions
----------------------------------------------------------------------------------------------------*/

/*----
	[list of users      ]

	name   [   ]
	passwd [   ]

	<ins> <delete> <update> <quit>

-----*/

enum {
	MAINT_USER_LIST,
	MAINT_USER_NAME,
	MAINT_USER_PASSWD,

	MAINT_USER_INS_BTN,
	MAINT_USER_DEL_BTN,
	MAINT_USER_MOD_BTN,
	MAINT_USER_QUIT_BTN
};

static void user_list_populate (GtkCList *user_list);

static void maint_user_callback (GtkWidget *widget, gpointer data)
{
	DataEntry *data_entry;
	GtkInput *name, *passwd;
	GtkToggleButton *insert_btn;
	GtkCList *user_list;
	int btn_num;
	char command[255];

	data_entry = (DataEntry *) data;
	btn_num = data_entry_get_widget_id (data_entry, widget);

	user_list = GTK_CLIST(data_entry_get_widget (data_entry, MAINT_USER_LIST));
	name = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_USER_NAME));
	passwd = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_USER_PASSWD));
	insert_btn = GTK_TOGGLE_BUTTON(data_entry_get_widget (data_entry, MAINT_USER_INS_BTN));

	switch (btn_num)
	{
		case MAINT_USER_MOD_BTN:
			if (insert_btn->active)
			{
				sprintf (command, "insert into users values ('%s','%s')",
					gtk_input_get_text (name), gtk_input_get_text (passwd));
			}
			else // modify
			{
				sprintf (command, "update users set username = '%s', passwd = '%s' where username = '%s'",
					gtk_input_get_text (name), gtk_input_get_text (passwd),
					(char *) gtk_object_get_user_data (GTK_OBJECT(name)));
			}
			//printf ("Command = '%s'\n", command);
			res = PQexec (conn, command);
			PQclear (res);
			gtk_input_set_text (name, "");
			gtk_input_set_text (passwd, "");
			gtk_widget_grab_focus (GTK_WIDGET(user_list));
			gtk_toggle_button_set_state (insert_btn, FALSE);
			user_list_populate (user_list);
			break;

		case MAINT_USER_INS_BTN:
			gtk_widget_grab_focus (GTK_WIDGET(name));
			break;

		case MAINT_USER_DEL_BTN:
			// delete the selected record
			sprintf (command, "delete from users where username = '%s'", 
				gtk_input_get_text (name));
			//printf ("Command = '%s'\n", command);
			res = PQexec (conn, command);
			PQclear (res);
			user_list_populate (user_list);
			gtk_input_set_text (name, "");
			gtk_input_set_text (passwd, "");
			gtk_widget_grab_focus (GTK_WIDGET(user_list));
			gtk_toggle_button_set_state (insert_btn, FALSE);
			break;

		case MAINT_USER_QUIT_BTN:
			gtk_widget_hide (GTK_WIDGET(data_entry));
			break;
	}
}

static void user_list_populate (GtkCList *user_list)
{
	int i, nTuples;
	char *user_list_row[2];

	gtk_clist_freeze (user_list);
	gtk_clist_clear (user_list);
    //printf ("select username,passwd from users");
    res = PQexec(conn, "select username,passwd from users order by username");
    if (PQresultStatus(res) != PGRES_TUPLES_OK)
    {
        fprintf(stderr, "\nSelect didn't return tuples properly\n");
        PQclear(res);
        exit_nicely(conn);
    }

	nTuples = PQntuples (res);
	for (i = 0; i < nTuples; i++)
	{
		user_list_row[0] = PQgetvalue (res, i, 0);
		user_list_row[1] = PQgetvalue (res, i, 1);
		gtk_clist_append (GTK_CLIST(user_list), user_list_row);
	}
	gtk_clist_thaw (user_list);
	PQclear (res);
}

static void maint_user_list_select (GtkWidget *widget, gint row, gint col, GdkEventButton *event, gpointer data)
{
	DataEntry *data_entry;
	GtkInput *name, *passwd;
	GtkToggleButton *insert_btn;
	gchar *text;

	g_return_if_fail (widget);
	g_return_if_fail (data);
	data_entry = DATA_ENTRY (data);

	name = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_USER_NAME));
	passwd = GTK_INPUT(data_entry_get_widget (data_entry, MAINT_USER_PASSWD));
	insert_btn = GTK_TOGGLE_BUTTON(data_entry_get_widget (data_entry, MAINT_USER_INS_BTN));

	gtk_clist_get_text (GTK_CLIST(widget), row, 0, &text);
	gtk_input_set_text (name, text);
	gtk_object_set_user_data (GTK_OBJECT(name), text);
	gtk_clist_get_text (GTK_CLIST(widget), row, 1, &text);
	gtk_input_set_text (passwd, text);
	gtk_widget_grab_focus (GTK_WIDGET(name));

	gtk_toggle_button_set_state (insert_btn, FALSE);
}

void maint_users (GtkWidget *widget, gpointer data)
{
	DataEntry *data_entry;
	static GtkWidget *user_list, *name, *passwd, *delbtn;
	GtkAcceleratorTable *accel;
	static char *titles[] = 
	{
		"Name", "Password"
	};

	if (!maint_user_data_entry)
	{
		maint_user_data_entry = DATA_ENTRY(data_entry_new ("Users", 20, 140, 0,0));
		data_entry = maint_user_data_entry;

		accel = gtk_accelerator_table_new ();
		gtk_window_add_accelerator_table (GTK_WINDOW(data_entry), accel);

		user_list = gtk_clist_new_with_titles (2, titles);
		gtk_clist_set_column_width (GTK_CLIST(user_list), 0, 100);
		gtk_widget_set_usize (GTK_WIDGET(user_list), 200, 200);
		gtk_signal_connect (GTK_OBJECT (user_list), "select_row", 
			(GtkSignalFunc) maint_user_list_select, data_entry);

		data_entry_add_widget (data_entry, MAINT_USER_LIST, user_list, 0,0,10,1);

		data_entry_add_label_button (data_entry, MAINT_USER_INS_BTN, "Insert", 
				GTK_SIGNAL_FUNC(maint_user_callback), BUTTON_NORMAL | BUTTON_TOGGLE);

		delbtn = data_entry_add_label_button (data_entry, MAINT_USER_DEL_BTN, "Delete", 
				GTK_SIGNAL_FUNC(maint_user_callback), BUTTON_NORMAL);
		gtk_accelerator_table_install (accel, GTK_OBJECT(delbtn), "clicked", '', 0);

		data_entry_add_label_button (data_entry, MAINT_USER_MOD_BTN, "Update", 
				GTK_SIGNAL_FUNC(maint_user_callback), BUTTON_DEFAULT);
		data_entry_add_label_button (data_entry, MAINT_USER_QUIT_BTN, "Quit", 
				GTK_SIGNAL_FUNC(maint_user_callback), BUTTON_QUIT);

		// now add the fields
		name = data_entry_add_field (data_entry, MAINT_USER_NAME, "Name:", 
				32, INPUT_UPPER, 1,11,1,1);
		passwd = data_entry_add_field (data_entry, MAINT_USER_PASSWD, "Password:",
				32, INPUT_NORMAL, 1,12,1,1);

		gtk_widget_grab_focus (name);
	}

	if (!GTK_WIDGET_VISIBLE(maint_user_data_entry))
		gtk_widget_show (GTK_WIDGET(maint_user_data_entry));
	user_list_populate (GTK_CLIST(user_list));

}
